/* This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/



//FileTreeModel.java - this class is passed the address of a directory and displays the
//contents in a tree structure,
//This class is used with permission from the Sun.Java site http://www.java.sun.com

package com.misc;



import javax.swing.event.TreeModelListener;
import javax.swing.event.TreeModelEvent;

import java.io.File;

import javax.swing.tree.*;






/**
 * The methods in this class allow the JTree component to traverse
 * the file system tree, and display the files and directories.
 **/
	public class FileTreeModel implements TreeModel {
  // We specify the root directory when we create the model.
		protected File root;
		public FileTreeModel(File root) { this.root = root; }

  // The model knows how to return the root object of the tree
		public Object getRoot() { return root; }

  // Tell JTree whether an object in the tree is a leaf or not
		public boolean isLeaf(Object node) {  return ((File)node).isFile(); }

  // Tell JTree how many children a node has
		public int getChildCount(Object parent) {
			String[] children = ((File)parent).list();
			if (children == null) return 0;
			return children.length;
		}

  // Fetch any numbered child of a node for the JTree.
  // Our model returns File objects for all nodes in the tree.  The
  // JTree displays these by calling the File.toString() method.
		public Object getChild(Object parent, int index) {
			String[] children = ((File)parent).list();
			if ((children == null) || (index >= children.length)) return null;
			return new File((File) parent, children[index]);
		}

  // Figure out a child's position in its parent node.
		public int getIndexOfChild(Object parent, Object child) {
			String[] children = ((File)parent).list();
			if (children == null) return -1;
			String childname = ((File)child).getName();
			for(int i = 0; i < children.length; i++) {
				if (childname.equals(children[i])) return i;
			}
			return -1;
		}
		

  // This method is only invoked by the JTree for editable trees.
  // This TreeModel does not allow editing, so we do not implement
  // this method.  The JTree editable property is false by default.
		public void valueForPathChanged(TreePath path, Object newvalue) {}

  // Since this is not an editable tree model, we never fire any events,
  // so we don't actually have to keep track of interested listeners.
		public void addTreeModelListener(TreeModelListener l) {}
		public void removeTreeModelListener(TreeModelListener l) {}


	}


