/*
 * Copyright (C) S.Chekanov  
 *
 * This cxxram is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This cxxram is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this cxxram; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.syntax;

import com.utils.*;
import java.net.URL;

import java.io.*;
import javax.swing.text.Segment;

/**
 * C token marker.
 *
 * @author Slava Pestov
 * @version $Id: CXXTokenMarker.java,v 1.35 2000/01/29 10:12:43 sp Exp $
 */
public class CXXTokenMarker extends TokenMarker
{

        public static final String METHOD_DELIMITERS = " \t~!%^*()-+=|\\#/{}[]:;\"'<>,.?@";

	public CXXTokenMarker()
	{
		this(true,false,getKeywords());
		cpp = false;
		javadoc = false;
	}

	public CXXTokenMarker(boolean cpp, boolean javadoc, KeywordMap keywords)
	{
		this.cpp = cpp;
		this.javadoc = javadoc;
		this.keywords = keywords;
		cpp = false;
		javadoc = false;
	}

	public byte markTokensImpl(byte token, Segment line, int lineIndex)
	{
		char[] array = line.array;
		int offset = line.offset;
		lastOffset = offset;
		lastKeyword = offset;
		int length = line.count + offset;
		boolean backslash = false;

loop:		for(int i = offset; i < length; i++)
		{
			int i1 = (i+1);

			char c = array[i];
			if(c == '\\')
			{
				backslash = !backslash;
				continue;
			}

			switch(token)
			{
			case Token.NULL:
				switch(c)
				{
				case '#':
					if(backslash)
						backslash = false;
					else if(cpp)
					{
						if(doKeyword(line,i,c))
							break;
						addToken(i - lastOffset,token);
						addToken(length - i,Token.KEYWORD2);
						lastOffset = lastKeyword = length;
						break loop;
					}
					break;
				case '"':
					doKeyword(line,i,c);
					if(backslash)
						backslash = false;
					else
					{
						addToken(i - lastOffset,token);
						token = Token.LITERAL1;
						lastOffset = lastKeyword = i;
					}
					break;
				case '\'':
					doKeyword(line,i,c);
					if(backslash)
						backslash = false;
					else
					{
						addToken(i - lastOffset,token);
						token = Token.LITERAL2;
						lastOffset = lastKeyword = i;
					}
					break;
				case ':':
					if(lastKeyword == offset)
					{
						if(doKeyword(line,i,c))
							break;
						backslash = false;
						addToken(i1 - lastOffset,Token.LABEL);
						lastOffset = lastKeyword = i1;
					}
					else if(doKeyword(line,i,c))
						break;
					break;
				case '/':
					backslash = false;
					doKeyword(line,i,c);
					if(length - i > 1)
					{
						switch(array[i1])
						{
						case '*':
							addToken(i - lastOffset,token);
							lastOffset = lastKeyword = i;
							if(javadoc && length - i > 2
								&& array[i+2] == '*')
								token = Token.COMMENT2;
							else
								token = Token.COMMENT1;
							break;
						case '/':
							addToken(i - lastOffset,token);
							addToken(length - i,Token.COMMENT1);
							lastOffset = lastKeyword = length;
							break loop;
						}
					}
					break;



                              default:
                                backslash = false;
                                if (!Character.isLetterOrDigit(c) && c != '_')
                                doKeyword(line, i, c);
                                if (METHOD_DELIMITERS.indexOf(c) != -1)
                                {
                                lastWhitespace = i;
                                }
                             break;

				}
				break;
			case Token.COMMENT1:
			case Token.COMMENT2:
				backslash = false;
				if(c == '*' && length - i > 1)
				{
					if(array[i1] == '/')
					{
						i++;
						addToken((i+1) - lastOffset,token);
						token = Token.NULL;
						lastOffset = lastKeyword = i+1;
					}
				}
				break;
			case Token.LITERAL1:
				if(backslash)
					backslash = false;
				else if(c == '"')
				{
					addToken(i1 - lastOffset,token);
					token = Token.NULL;
					lastOffset = lastKeyword = i1;
				}
				break;
			case Token.LITERAL2:
				if(backslash)
					backslash = false;
				else if(c == '\'')
				{
					addToken(i1 - lastOffset,Token.LITERAL1);
					token = Token.NULL;
					lastOffset = lastKeyword = i1;
				}
				break;
			default:
				throw new InternalError("Invalid state: "
					+ token);
			}
		}

		if(token == Token.NULL)
			doKeyword(line,length,'\0');

		switch(token)
		{
		case Token.LITERAL1:
		case Token.LITERAL2:
			addToken(length - lastOffset,Token.INVALID);
			token = Token.NULL;
			break;
		case Token.KEYWORD2:
			addToken(length - lastOffset,token);
			if(!backslash)
				token = Token.NULL;
		default:
			addToken(length - lastOffset,token);
			break;
		}

		return token;
	}

	public static KeywordMap getKeywords()
	{
		if(cxxKeywords == null)
		{
			cxxKeywords = new KeywordMap(false);
			
			
			cxxKeywords.add("and", Token.KEYWORD3);
			cxxKeywords.add("and_eq", Token.KEYWORD3);
			cxxKeywords.add("asm", Token.KEYWORD2);         //
			cxxKeywords.add("auto", Token.KEYWORD1);        //
			cxxKeywords.add("bitand", Token.KEYWORD3);
			cxxKeywords.add("bitor", Token.KEYWORD3);
			cxxKeywords.add("bool",Token.KEYWORD3);
			cxxKeywords.add("break", Token.KEYWORD1);	   //
			cxxKeywords.add("case", Token.KEYWORD1);		   //
			cxxKeywords.add("catch", Token.KEYWORD1);
			cxxKeywords.add("char", Token.KEYWORD3);		   //
			cxxKeywords.add("class", Token.KEYWORD3);
			cxxKeywords.add("compl", Token.KEYWORD3);
			cxxKeywords.add("const", Token.KEYWORD1);	   //
			cxxKeywords.add("const_cast", Token.KEYWORD3);
			cxxKeywords.add("continue", Token.KEYWORD1);	   //
			cxxKeywords.add("default", Token.KEYWORD1);	   //
			cxxKeywords.add("delete", Token.KEYWORD1);
                        cxxKeywords.add("cout", Token.KEYWORD1);
                        cxxKeywords.add("endl", Token.KEYWORD1);
                        cxxKeywords.add("std", Token.KEYWORD1);
                        cxxKeywords.add("ofstream", Token.KEYWORD1);
                        cxxKeywords.add("ifstream", Token.KEYWORD1);
                        cxxKeywords.add("fopen", Token.KEYWORD1);
                        cxxKeywords.add("fclose", Token.KEYWORD1);
                        cxxKeywords.add("close", Token.KEYWORD1);
                        cxxKeywords.add("sprintf", Token.KEYWORD1);
                        cxxKeywords.add("printf", Token.KEYWORD1);
                        cxxKeywords.add("strcmp", Token.KEYWORD1);
                        cxxKeywords.add("getline", Token.KEYWORD1);
                        cxxKeywords.add("ignore", Token.KEYWORD1);
                        cxxKeywords.add("get", Token.KEYWORD1);
                        cxxKeywords.add("read", Token.KEYWORD1);
                        cxxKeywords.add("peek", Token.KEYWORD1);
                        cxxKeywords.add("string", Token.KEYWORD3);
                        cxxKeywords.add("vector", Token.KEYWORD3);
              		cxxKeywords.add("do",Token.KEYWORD1);           
			cxxKeywords.add("double" ,Token.KEYWORD3);	   
			cxxKeywords.add("dynamic_cast", Token.KEYWORD3);
			cxxKeywords.add("else", 	Token.KEYWORD1);	   
			cxxKeywords.add("enum",  Token.KEYWORD3);	   
			cxxKeywords.add("explicit", Token.KEYWORD1);			
			cxxKeywords.add("export", Token.KEYWORD2);
			cxxKeywords.add("extern", Token.KEYWORD2);	   
			cxxKeywords.add("false", Token.LITERAL2);
			cxxKeywords.add("float", Token.KEYWORD3);	   
                        cxxKeywords.add("Int_t", Token.KEYWORD3);
			cxxKeywords.add("for", Token.KEYWORD1);		   
			cxxKeywords.add("friend", Token.KEYWORD1);			
			cxxKeywords.add("goto", Token.KEYWORD1);       
			cxxKeywords.add("if", Token.KEYWORD1);		   
			cxxKeywords.add("inline", Token.KEYWORD1);
			cxxKeywords.add("int", Token.KEYWORD3);		   
			cxxKeywords.add("long", Token.KEYWORD3);		
			cxxKeywords.add("mutable", Token.KEYWORD3);
			cxxKeywords.add("namespace", Token.KEYWORD3);
			cxxKeywords.add("not", Token.KEYWORD3);
			cxxKeywords.add("not_eq", Token.KEYWORD3);
			cxxKeywords.add("operator", Token.KEYWORD3);
			cxxKeywords.add("or", Token.KEYWORD3);
			cxxKeywords.add("or_eq", Token.KEYWORD3);
			cxxKeywords.add("private", Token.KEYWORD1);
			cxxKeywords.add("protected", Token.KEYWORD1);
			cxxKeywords.add("public", Token.KEYWORD1);
			cxxKeywords.add("register", Token.KEYWORD1);
			cxxKeywords.add("reinterpret_cast", Token.KEYWORD3);
			cxxKeywords.add("return", Token.KEYWORD1);      
			cxxKeywords.add("short", Token.KEYWORD3);	  
			cxxKeywords.add("signed", Token.KEYWORD3);	  
			cxxKeywords.add("sizeof", Token.KEYWORD1);	  
			cxxKeywords.add("static", Token.KEYWORD1);	   
			cxxKeywords.add("static_cast", Token.KEYWORD3);
			cxxKeywords.add("struct", Token.KEYWORD3);	  
			cxxKeywords.add("switch", Token.KEYWORD1);	   
			cxxKeywords.add("template", Token.KEYWORD3);
			cxxKeywords.add("this", Token.LITERAL2);
			cxxKeywords.add("throw", Token.KEYWORD1);
			cxxKeywords.add("true", Token.LITERAL2);
			cxxKeywords.add("try", Token.KEYWORD1);
			cxxKeywords.add("typedef", Token.KEYWORD3);	  
			cxxKeywords.add("typeid", Token.KEYWORD3);
			cxxKeywords.add("typename", Token.KEYWORD3);
			cxxKeywords.add("union", Token.KEYWORD3);	   
			cxxKeywords.add("unsigned", Token.KEYWORD3);	  
			cxxKeywords.add("using", Token.KEYWORD2);
			cxxKeywords.add("virtual", Token.KEYWORD1);
			cxxKeywords.add("void", Token.KEYWORD1);		  
			cxxKeywords.add("volatile", Token.KEYWORD1);	   
			cxxKeywords.add("wchar_t", Token.KEYWORD3);
			cxxKeywords.add("while", Token.KEYWORD1);	   
			cxxKeywords.add("xor", Token.KEYWORD3);
			cxxKeywords.add("xor_eq", Token.KEYWORD3);            


			// non ANSI keywords
      cxxKeywords.add("NULL", Token.LITERAL2);

      cxxKeywords.add("new", Token.METHOD);
// preprocess

      cxxKeywords.add("#include", Token.OPERATOR);
      cxxKeywords.add("#define", Token.METHOD);
      cxxKeywords.add("#undef", Token.METHOD);
      cxxKeywords.add("#ifndef", Token.METHOD);
      cxxKeywords.add("#endif", Token.METHOD);
      cxxKeywords.add("#else", Token.METHOD);
      cxxKeywords.add("#if", Token.METHOD);
      cxxKeywords.add("#elif", Token.METHOD);
      cxxKeywords.add("#efined", Token.METHOD);
      cxxKeywords.add("#line", Token.METHOD);
      cxxKeywords.add("#error", Token.METHOD);
      cxxKeywords.add("#pragma", Token.METHOD);



                          cxxKeywords.add("Double_t", Token.KEYWORD3);
			  cxxKeywords.add("Float_t", Token.KEYWORD3);
			  cxxKeywords.add("Int_t", Token.KEYWORD3);
                          cxxKeywords.add("UInt_t", Token.KEYWORD3);
                          cxxKeywords.add("Char_t", Token.KEYWORD3);
                          cxxKeywords.add("Bool_t", Token.KEYWORD3);
                          cxxKeywords.add("Short_t", Token.KEYWORD3);
                          cxxKeywords.add("Long64_t", Token.KEYWORD3);
                          cxxKeywords.add("ULong64_t", Token.KEYWORD3);
                          cxxKeywords.add("Double32_t", Token.KEYWORD3);

// Global
                        cxxKeywords.add("gROOT", Token.KEYWORD1);
                        cxxKeywords.add("gStyle", Token.KEYWORD1);
                        cxxKeywords.add("gPad", Token.KEYWORD1);
                        cxxKeywords.add("gSystem", Token.KEYWORD1);
                        cxxKeywords.add("gRandom", Token.KEYWORD1);
                        cxxKeywords.add("gBenchmark", Token.KEYWORD1);
                        cxxKeywords.add("gMinuit", Token.KEYWORD1);
                        cxxKeywords.add("gEnv", Token.KEYWORD1);
                        cxxKeywords.add("gDirectory", Token.KEYWORD1);
                        cxxKeywords.add("gFile", Token.KEYWORD1);
                        cxxKeywords.add("gHTML", Token.KEYWORD1);


// read ROOT methods and classes 
        ReadRoot();


		}
		return cxxKeywords;
	}




// read ROOT 
 private static void ReadRoot() {

    String thisLine;
//    System.out.println("Read ROOT files");
// load classes 
      URL  sfile1 =  ResourceLoader.getResourceURL( "#standards/root_syntax/classes.d" );
     try {
        BufferedReader in = new BufferedReader(
                                new InputStreamReader(
                                sfile1.openStream()));
      while ((thisLine = in.readLine()) != null) {
         if (thisLine.length()>1) {
                              // System.out.println(thisLine);
                              cxxKeywords.add(thisLine,Token.KEYWORD1); // end root classes
                              }
         } // end while
      }
    catch (Exception e) {
       System.out.println("Error in reading #standards/root_syntax/classes.d"); 
     //  e.printStackTrace();
      }


// load methods
    URL  sfile2 =  ResourceLoader.getResourceURL( "#standards/root_syntax/methods.d" );
     try {
        BufferedReader in = new BufferedReader(
                                new InputStreamReader(
                                sfile2.openStream()));
      while ((thisLine = in.readLine()) != null) {
         if (thisLine.length()>1) {
                              // System.out.println(thisLine);
                              cxxKeywords.add(thisLine,Token.KEYWORD2); // end root classes
                              }
         } // end while
      }
    catch (Exception e) {
       System.out.println("Error in reading #standards/root_syntax/methods.d"); 
       // e.printStackTrace();
      }


 
} // end read of root files 





// private members
  private static KeywordMap cxxKeywords;

  protected boolean cpp;
  protected boolean javadoc;
  protected KeywordMap keywords;
  protected int lastOffset;
  protected int lastKeyword;
  protected int lastWhitespace;

  protected boolean doKeyword(Segment line,  int i,  char c)
  {
    int i1 = i+1;

    int len = i - lastKeyword;
    byte id = keywords.lookup(line, lastKeyword, len);
    if (id != Token.NULL)
    {
      if (lastKeyword != lastOffset)
        addToken(lastKeyword - lastOffset, Token.NULL);
      addToken(len, id);
      lastOffset = i;
      lastKeyword = i1;
      lastWhitespace = i;
      return true;
    }
    lastKeyword = i1;
    return false;
  } 





}

/*
 * ChangeLog:
 * $Log: CXXTokenMarker.java,v $
 * Revision 1.35  2000/01/29 10:12:43  sp
 * BeanShell edit mode, bug fixes
 *
 * Revision 1.34  1999/12/13 03:40:29  sp
 * Bug fixes, syntax is now mostly GPL'd
 *
 * Revision 1.33  1999/10/31 07:15:34  sp
 * New logging API, splash screen updates, bug fixes
 *
 * Revision 1.32  1999/09/30 12:21:04  sp
 * No net access for a month... so here's one big jEdit 2.1pre1
 *
 * Revision 1.31  1999/08/21 01:48:18  sp
 * jEdit 2.0pre8
 *
 * Revision 1.30  1999/06/05 00:22:58  sp
 * LGPL'd syntax package
 *
 * Revision 1.29  1999/06/03 08:24:13  sp
 * Fixing broken CVS
 *
 * Revision 1.30  1999/05/31 08:11:10  sp
 * Syntax coloring updates, expand abbrev bug fix
 *
 * Revision 1.29  1999/05/31 04:38:51  sp
 * Syntax optimizations, HyperSearch for Selection added (Mike Dillon)
 *
 * Revision 1.28  1999/05/29 03:46:53  sp
 * CXXTokenMarker bug fix, new splash screen
 *
 * Revision 1.27  1999/05/14 04:56:15  sp
 * Docs updated, default: fix in C/C++/Java mode, full path in title bar toggle
 *
 * Revision 1.26  1999/05/11 09:05:10  sp
 * New version1.6.html file, some other stuff perhaps
 *
 * Revision 1.25  1999/04/22 06:03:26  sp
 * Syntax colorizing change
 *
 * Revision 1.24  1999/04/19 05:38:20  sp
 * Syntax API changes
 *
 * Revision 1.23  1999/03/13 08:50:39  sp
 * Syntax colorizing updates and cleanups, general code reorganizations
 *
 * Revision 1.22  1999/03/13 00:09:07  sp
 * Console updates, uncomment removed cos it's too buggy, cvs log tags added
 *
 * Revision 1.21  1999/03/12 23:51:00  sp
 * Console updates, uncomment removed cos it's too buggy, cvs log tags added
 *
 */
