C
C
      SUBROUTINE CHAN2A(BB)
C***********************************************************************
C
C     simple two channel model to realize low mass diffraction
C     (version A, iteration of triple- and loop-Pomeron)
C
C     input:     BB      impact parameter (mb**1/2)
C
C     output:    common /AMPLIT/
C                AMPEL      elastic amplitude
C                AMPVM(4,4) q-elastic VM production
C                AMLMSD(2)  low mass single diffraction amplitude
C                AMHMSD(2)  high mass single diffraction amplitude
C                AMLMDD     low mass double diffraction amplitude
C                AMHMDD     high mass double diffraction amplitude
C
C***********************************************************************
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      SAVE
C
      PARAMETER (GEV2MB=0.38935)
      PARAMETER (PI=3.141592654,
     &           DEPS=1.D-5,
     &           ZERO=0.D0,
     &           TWO=2.D0,
     &           FOUR=4.D0,
     &           EIGHT=8.D0,
     &           ONE=1.D0)
C
      PARAMETER (NMAXD=100)
      COMMON /DEBUG/ IDEB(NMAXD),KSPOM,KHPOM,KSREG,KHDIR,KACCEP,
     &  KSTRG,KHTRG,KSLOO,KHLOO,KSDPO,KHDPO,KEVENT,KSOFT,KHARD
C
      CHARACTER*8 MDLNA
      COMMON /MODELS/ MDLNA(50),ISWMDL(50),PARMDL(200),IPAMDL(200)
C
C  eikonal functions and slopes
      PARAMETER(MDIM=5)
      COMMON /ZXEIKS/ XM(MDIM,MDIM),XOUT(MDIM,MDIM),
     &                XIN(MDIM),XOU(MDIM),
     &                ZXP(MDIM,MDIM),BXP(MDIM,MDIM),
     &                ZXR(MDIM,MDIM),BXR(MDIM,MDIM),
     &                ZXH(MDIM,MDIM),BXH(MDIM,MDIM),
     &                ZXD(MDIM,MDIM),BXD(MDIM,MDIM),
     &                ZXT1A(MDIM,MDIM),BXT1A(MDIM,MDIM),
     &                ZXT1B(MDIM,MDIM),BXT1B(MDIM,MDIM),
     &                ZXT2A(MDIM,MDIM),BXT2A(MDIM,MDIM),
     &                ZXT2B(MDIM,MDIM),BXT2B(MDIM,MDIM),
     &                ZXL(MDIM,MDIM),BXL(MDIM,MDIM),
     &                ZXDP(MDIM,MDIM),BXDP(MDIM,MDIM),
     &                ZXD1A(MDIM,MDIM),BXD1A(MDIM,MDIM),
     &                ZXD1B(MDIM,MDIM),BXD1B(MDIM,MDIM),
     &                ZXD2A(MDIM,MDIM),BXD2A(MDIM,MDIM),
     &                ZXD2B(MDIM,MDIM),BXD2B(MDIM,MDIM)
C
C  output common
      COMPLEX*16      AMPEL,AMPVM,AMPSOF,AMPHAR,AMLMSD,AMHMSD,AMLMDD,
     &                AMHMDD,AMPDP,AMPD1,AMPD2
      COMMON /AMPLIT/ AMPEL,AMPVM(4,4),AMPSOF,AMPHAR,AMLMSD(2),
     &                AMHMSD(2),AMLMDD,AMHMDD,AMPDP,AMPD1,AMPD2
C
      COMMON /REGGE/  ALPOM,ALPOMP,GP(2),B0POM(2),
     &                ALREG,ALREGP,GR(2),B0REG(2),
     &                GPPP,GPPR,B0PPP,B0PPR,
     &                VDMFAC(4),VDMQ2F(4),B0HAR,AKFAC
      COMMON /TWOCHA/ PHISUP(2),RMASS1,RMASS2,VAR
      COMMON /GLOCMS/ ECM,PCM,PMASS(2),PVIRT(2),IFPAP(2),IFPAB(2)
C
C  local variables
      DIMENSION  AB(6,4),CHI(4),CHDS(4),CHDH(4),CHDA(4),CHDB(4),
     &           CHDD(4),CHDP(4),
     &           AMPCHA(4),EX1CHI(4),EX2CHI(4),ABSUM(4),AMPELA(4,7),
     &           ELAFAC(4),AMPFAC(4)
      DIMENSION CHIFAC(4,4),EXPFAC(4,4),IELTAB(4,4)
C  combinatorical factors
      DATA      CHIFAC / 1.D0, 1.D0,-1.D0,-1.D0,
     &                   1.D0,-1.D0, 1.D0,-1.D0,
     &                   1.D0,-1.D0,-1.D0, 1.D0,
     &                   1.D0, 1.D0, 1.D0, 1.D0 /
      DATA      EXPFAC / 1.D0, 1.D0, 1.D0, 1.D0,
     &                   1.D0,-1.D0,-1.D0, 1.D0,
     &                  -1.D0, 1.D0,-1.D0, 1.D0,
     &                  -1.D0,-1.D0, 1.D0, 1.D0 /
      DATA      IELTAB / 1, 2, 3, 4,
     &                   2, 1, 4, 3,
     &                   3, 4, 1, 2,
     &                   4, 3, 2, 1 /
C
      IF(IDEB(86).GE.20) THEN
        WRITE(6,'(1X,A,E12.3)') 'CHAN2A:DEBUG:BB ',BB
      ENDIF
      B24 = BB**2/FOUR
C  VDM factors
      AMPFAC(1) = SQRT(VDMQ2F(1)*VDMQ2F(3))
      AMPFAC(2) = SQRT(VDMQ2F(2)*VDMQ2F(3))
      AMPFAC(3) = SQRT(VDMQ2F(1)*VDMQ2F(4))
      AMPFAC(4) = SQRT(VDMQ2F(2)*VDMQ2F(4))
      ELAFAC(1) = (VDMQ2F(1)*VDMQ2F(3)+VDMQ2F(2)*VDMQ2F(3)
     &             +VDMQ2F(1)*VDMQ2F(4)+VDMQ2F(2)*VDMQ2F(4))/EIGHT
      ELAFAC(2) = (AMPFAC(1)*AMPFAC(2)+AMPFAC(3)*AMPFAC(4))/FOUR
      ELAFAC(3) = (AMPFAC(1)*AMPFAC(3)+AMPFAC(2)*AMPFAC(4))/FOUR
      ELAFAC(4) = AMPFAC(1)*AMPFAC(4)/TWO
C
      DO 25 I=1,4
        AB(1,I)=ZXP(1,I)*EXP(-B24/BXP(1,I))
     &          +ZXR(1,I)*EXP(-B24/BXR(1,I))
        AB(2,I)=ZXH(1,I)*EXP(-B24/BXH(1,I))
        AB(3,I)=-ZXT1A(1,I)*EXP(-B24/BXT1A(1,I))
        AB(4,I)=-ZXT2A(1,I)*EXP(-B24/BXT2A(1,I))
        AB(5,I)=-ZXL(1,I)*EXP(-B24/BXL(1,I))
     &          -ZXT1B(1,I)*EXP(-B24/BXT1B(1,I))
     &          -ZXT2B(1,I)*EXP(-B24/BXT2B(1,I))
        AB(6,I)=ZXDP(1,I)*EXP(-B24/BXDP(1,I))
 25   CONTINUE
C
      DO 50 I=1,4
        ABSUM(I)  = ZERO
        DO 75 II=6,1,-1
          ABSUM(I) = ABSUM(I) + AB(II,I)
 75     CONTINUE
 50   CONTINUE
      IF(IDEB(86).GE.20) THEN
        WRITE(6,'(1X,A,4E12.3)') 'CHAN2A:DEBUG:ABSUM',ABSUM
      ENDIF
      DO 100 I=1,4
        CHI(I)  = ZERO
        CHDS(I) = ZERO
        CHDH(I) = ZERO
        CHDA(I) = ZERO
        CHDB(I) = ZERO
        CHDD(I) = ZERO
        CHDP(I) = ZERO
        AMPELA(I,5) = ZERO
        AMPELA(I,6) = ZERO
        AMPELA(I,7) = ZERO
        DO 200 K=1,4
          AMPELA(I,K) = ZERO
          AMPVM(I,K)  = ZERO
          CHI(I)  = CHI(I)  + CHIFAC(K,I)*ABSUM(K)
          CHDS(I) = CHDS(I) + CHIFAC(K,I)*AB(1,K)
          CHDH(I) = CHDH(I) + CHIFAC(K,I)*AB(2,K)
          CHDA(I) = CHDA(I) + CHIFAC(K,I)*AB(3,K)
          CHDB(I) = CHDB(I) + CHIFAC(K,I)*AB(4,K)
          CHDD(I) = CHDD(I) + CHIFAC(K,I)*AB(5,K)
          CHDP(I) = CHDP(I) + CHIFAC(K,I)*AB(6,K)
 200    CONTINUE
        IF(CHI(I).LT.-DEPS) THEN
          IF(IDEB(86).GE.0) THEN
            WRITE(6,'(1X,A,I3,2E12.3)')
     &        'CHAN2A:WARNING: NEG.EIGENVALUE:(I,B,CHI)',
     &        I,BB,CHI(I)
            WRITE(6,'(5X,A,5E12.3)') 'E,CHIs:',ECM,(ABSUM(K),K=1,4)
          ENDIF
        ENDIF
        IF(ABS(CHI(I)).GT.200.D0) THEN
          EX1CHI(I) = ZERO
          EX2CHI(I) = ZERO
        ELSE
          TMP       = EXP(-CHI(I))
          EX1CHI(I) = TMP
          EX2CHI(I) = TMP*TMP
        ENDIF
 100  CONTINUE
      IF(IDEB(86).GE.20) THEN
        WRITE(6,'(1X,A,4E12.3)') 'CHAN2A:DEBUG:EX1CHI',EX1CHI
      ENDIF
C
      AMPELA(1,1) = FOUR
      DO 300 K=1,4
        DO 400 J=1,4
          AMPELA(K,1) = AMPELA(K,1) - EXPFAC(J,K)*EX1CHI(J)
          AMPELA(K,2) = AMPELA(K,2)
     &      + EXPFAC(J,K)*TWO*CHDS(J)*EX2CHI(J)
          AMPELA(K,3) = AMPELA(K,3)
     &      + EXPFAC(J,K)*TWO*CHDH(J)*EX2CHI(J)
          AMPELA(K,4) = AMPELA(K,4)
     &      + EXPFAC(J,K)*(-TWO*(CHDA(J)+2.D0*CHDP(J)))*EX2CHI(J)
          AMPELA(K,5) = AMPELA(K,5)
     &      + EXPFAC(J,K)*(-TWO*(CHDB(J)+2.D0*CHDP(J)))*EX2CHI(J)
          AMPELA(K,6) = AMPELA(K,6)
     &      + EXPFAC(J,K)*(-TWO*CHDD(J))*EX2CHI(J)
          AMPELA(K,7) = AMPELA(K,7)
     &      + EXPFAC(J,K)*TWO*CHDP(J)*EX2CHI(J)
 400    CONTINUE
 300  CONTINUE
C
      IF(IDEB(86).GE.25) THEN
        DO 305 I=1,7
          WRITE(6,'(1X,A,I3,4E10.3)') 'CHAN2A:DEBUG:AMPELA(1-4,I)',I,
     &      (AMPELA(K,1),K=1,4)
 305    CONTINUE
      ENDIF
C
C  VDM factors --> amplitudes
C  low mass excitations
      DO 500 I=1,4
        AMPCHA(I) = ZERO
        DO 600 K=1,4
          AMPCHA(I) = AMPCHA(I) + AMPFAC(K)*AMPELA(IELTAB(K,I),1)
 600    CONTINUE
 500  CONTINUE
      AMPVME    = AMPCHA(1)/EIGHT
      AMLMSD(1) = AMPCHA(2)/EIGHT
      AMLMSD(2) = AMPCHA(3)/EIGHT
      AMLMDD    = AMPCHA(4)/EIGHT
C  elastic part, high mass diffraction
      AMPEL = 0.5D0*ZXD(1,1)*EXP(-B24/BXD(1,1))
      AMPSOF    = ZERO
      AMPHAR    = ZERO
      AMHMSD(1) = ZERO
      AMHMSD(2) = ZERO
      AMHMDD    = ZERO
      AMPDP     = ZERO
      DO 450 I=1,4
        AMPEL     = AMPEL     + ELAFAC(I)*AMPELA(I,1)
        AMPSOF    = AMPSOF    + ELAFAC(I)*AMPELA(I,2)
        AMPHAR    = AMPHAR    + ELAFAC(I)*AMPELA(I,3)
        AMHMSD(1) = AMHMSD(1) + ELAFAC(I)*AMPELA(I,4)
        AMHMSD(2) = AMHMSD(2) + ELAFAC(I)*AMPELA(I,5)
        AMHMDD    = AMHMDD    + ELAFAC(I)*AMPELA(I,6)
        AMPDP     = AMPDP     + ELAFAC(I)*AMPELA(I,7)
 450  CONTINUE
      AMPSOF    = AMPSOF/TWO
      AMPHAR    = AMPHAR/TWO
      AMHMSD(1) = AMHMSD(1)/TWO
      IF(DREAL(AMHMSD(1)).LE.ZERO) AMHMSD(1) = ZERO
      AMHMSD(2) = AMHMSD(2)/TWO
      IF(DREAL(AMHMSD(2)).LE.ZERO) AMHMSD(2) = ZERO
      AMHMDD    = AMHMDD/TWO
      IF(DREAL(AMHMDD).LE.ZERO) AMHMDD = ZERO
      AMPDP     = AMPDP/TWO
      IF(DREAL(AMPDP).LE.ZERO) AMPDP = ZERO
C  vector-meson production, weight factors
      IF((IFPAP(1).EQ.22).OR.(IFPAP(2).EQ.22)) THEN
        IF(IFPAP(1).EQ.22) THEN
          IF(IFPAP(2).EQ.22) THEN
            DO 10 I=1,4
              DO 15 J=1,4
                AMPVM(I,J) = PARMDL(9+I)*PARMDL(9+J)*AMPVME
 15           CONTINUE
 10         CONTINUE
          ELSE
            AMPVM(1,1) = PARMDL(10)*AMPVME
            AMPVM(2,1) = PARMDL(11)*AMPVME
            AMPVM(3,1) = PARMDL(12)*AMPVME
            AMPVM(4,1) = PARMDL(13)*AMPVME
          ENDIF
        ELSE IF(IFPAP(2).EQ.22) THEN
          AMPVM(1,1) = PARMDL(10)*AMPVME
          AMPVM(1,2) = PARMDL(11)*AMPVME
          AMPVM(1,3) = PARMDL(12)*AMPVME
          AMPVM(1,4) = PARMDL(13)*AMPVME
        ENDIF
      ENDIF
C  debug output
      IF(IDEB(86).GE.5) THEN
        WRITE(6,'(/,1X,A)') 'CHAN2A:DEBUG:IMPACT PARAMETER AMPLITUDES:'
        WRITE(6,'(1X,A,2E12.3)') '       AMPEL',AMPEL
        WRITE(6,'(1X,A,8E10.3)') 'AMPVM(1,1-4)',(AMPVM(1,K),K=1,4)
        WRITE(6,'(1X,A,8E10.3)') 'AMPVM(2,1-4)',(AMPVM(2,K),K=1,4)
        WRITE(6,'(1X,A,8E10.3)') 'AMPVM(3,1-4)',(AMPVM(3,K),K=1,4)
        WRITE(6,'(1X,A,8E10.3)') 'AMPVM(4,1-4)',(AMPVM(4,K),K=1,4)
        WRITE(6,'(1X,A,4E12.3)') '  AMPSOF/HAR',AMPSOF,AMPHAR
        WRITE(6,'(1X,A,4E12.3)') '      AMLMSD',AMLMSD
        WRITE(6,'(1X,A,4E12.3)') '      AMHMSD',AMHMSD
        WRITE(6,'(1X,A,2E12.3)') '      AMLMDD',AMLMDD
        WRITE(6,'(1X,A,2E12.3)') '      AMHMDD',AMHMDD
        WRITE(6,'(1X,A,2E12.3)') '       AMPDP',AMPDP
      ENDIF
      END
