C
C
      SUBROUTINE GGEPEM(NEVENT,EE1,EE2)
C**********************************************************************
C
C     interface to call PHOJET (variable energy run) for
C     gamma-gamma collisions on e+e- collider
C
C     fully differential equivalent photon approximation 
C     to get photon flux
C
C     input:     EE1     LAB system energy of electron/positron 1
C                EE2     LAB system energy of electron/positron 2
C                NEVENT  number of events to generate
C            from COMMON /LEPCUT/:
C                YMIN1   lower limit of Y1
C                        (energy fraction taken by photon from electron)
C                YMAX1   upper limit of Y1
C                Q2MIN1  lower limit of photon virtuality
C                Q2MAX1  upper limit of photon virtuality
C                THMIN1  lower limit of scattered electron
C                THMAX1  upper limit of scattered electron
C                YMIN2   lower limit of Y2
C                        (energy fraction taken by photon from electron)
C                YMAX2   upper limit of Y2
C                Q2MIN2  lower limit of photon virtuality
C                Q2MAX2  upper limit of photon virtuality
C                THMIN2  lower limit of scattered electron
C                THMAX2  upper limit of scattered electron
C
C**********************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      SAVE
C
      PARAMETER ( ZERO = 0.D0,
     &            ONE  = 1.D0,
     &            TWO  = 2.D0,
     &            PI   = 3.14159265359D0 )
C
C  lepton/hadron-photon kinematics
      COMMON /PHOSRC/ PINI(5,2),PFIN(5,2),PGAM(5,2),
     &                GYY(2),GQ2(2),GGECM,GAIMP(2),PFTHE(2),PFPHI(2),
     &                IDPSRC(2),IDBSRC(2),RADSRC(2),AMSRC(2),GAMSRC(2)
      COMMON /LEPCUT/ ECMIN,ECMAX,EEMIN1,EEMIN2,
     &                YMIN1,YMAX1,YMIN2,YMAX2,
     &                Q2MIN1,Q2MAX1,Q2MIN2,Q2MAX2,
     &                THMIN1,THMAX1,THMIN2,THMAX2
C
      DIMENSION P1(4),P2(4)
      CHARACTER*72 TITLE
C
      WRITE(6,'(//1X,A,I10)') 'GGEPEM:DEBUG:EVENTS TO PROCESS',NEVENT
C  electron data
      ELEM = 0.512D-03
      ELEM2 = ELEM**2
C  lepton numbers
      IDPSRC(1) = 11
      IDPSRC(2) = -11
      IDBSRC(1) = 3
      IDBSRC(2) = 4
C  kinematic limitations
      YMI = ECMIN**2/(4.D0*EE1*EE2*YMAX2)
      IF(YMIN1.LT.YMI) THEN
        WRITE(6,'(/1X,A,2E12.5)')
     &    'GGEPEM:DEBUG:YMIN1 INCREASED TO (OLD/NEW)',YMIN1,YMI
        YMIN1 = YMI
      ELSE IF(YMIN1.GT.YMI) THEN
        WRITE(6,'(/1X,A,/1X,A,E12.5,A,E12.5)')
     &    'GGEPEM:WARNING:','ECM-CUT CORRESPONDS TO YMIN1 OF',YMI,
     &    '  INSTEAD OF',YMIN1
      ENDIF
      YMI = ECMIN**2/(4.D0*EE1*EE2*YMAX1)
      IF(YMIN2.LT.YMI) THEN
        WRITE(6,'(/1X,A,2E12.5)')
     &    'GGEPEM:DEBUG:YMIN2 INCREASED TO (OLD/NEW)',YMIN2,YMI
        YMIN2 = YMI
      ELSE IF(YMIN2.GT.YMI) THEN
        WRITE(6,'(/1X,A,/1X,A,E12.5,A,E12.5)')
     &    'GGEPEM:WARNING:','ECM-CUT CORRESPONDS TO YMIN2 OF',YMI,
     &    '  INSTEAD OF',YMIN2
      ENDIF
C
      X1MAX = LOG(YMAX1)
      X1MIN = LOG(YMIN1)
      X1DEL = X1MAX-X1MIN
      X2MAX = LOG(YMAX2)
      X2MIN = LOG(YMIN2)
      X2DEL = X2MAX-X2MIN
C
      MAXTAB = 50
      FLUX = 0.D0
      DELLY = LOG(YMAX1/YMIN1)/DBLE(MAXTAB-1)
*     WRITE(6,'(1X,A,I5)')'GGEPEM:DEBUG:TABLE OF PHOTON FLUX (SIDE 1)',
*    &  MAXTAB
      DO 100 I=1,MAXTAB
        Y1 = EXP(X1MIN+DELLY*DBLE(I-1))
        Q2LOW1 = MAX(Q2MIN1,ELEM2*Y1*Y1/(ONE-Y1))
        FF = ((ONE+(ONE-Y1)**2)/Y1*LOG(Q2MAX1/Q2LOW1)
     &        -TWO*ELEM2*Y1*(ONE/Q2LOW1-ONE/Q2MAX1))/(TWO*PI*137.D0)
        FLUX = FLUX+Y1*FF
*       WRITE(6,'(5X,2E15.4)') Y1,FF
 100  CONTINUE
      FLUX = FLUX*DELLY
      WRITE(6,'(1X,A,E12.4)') 
     &  'GGEPEM:DEBUG:INTEGRATED FLUX (ONE SIDE):',FLUX
C
      Q2LOW1 = MAX(Q2MIN1,ELEM2*YMIN1**2/(ONE-YMIN1))
      Q2LOW2 = MAX(Q2MIN2,ELEM2*YMIN2**2/(ONE-YMIN2))
      Y1 = YMIN1
      Y2 = YMIN2
      WGMAX = ((ONE+(ONE-Y1)**2)*LOG(Q2MAX1/Q2LOW1)
     &         -TWO*ELEM2*Y1*(ONE/Q2LOW1-ONE/Q2MAX1)*Y1)
     &       *((ONE+(ONE-Y2)**2)*LOG(Q2MAX2/Q2LOW2)
     &         -TWO*ELEM2*Y2*(ONE/Q2LOW2-ONE/Q2MAX2)*Y2)
C
C  photon 1
      EGAM = YMAX1*EE1
      P1(1) = ZERO
      P1(2) = ZERO
      P1(3) = EGAM
      P1(4) = EGAM
C  photon 2
      EGAM = YMAX2*EE2
      P2(1) = ZERO
      P2(2) = ZERO
      P2(3) = -EGAM
      P2(4) = EGAM
      CALL SETPAR(1,22,0,ZERO)
      CALL SETPAR(2,22,0,ZERO)
      CALL EVENT(-1,P1,P2,SIGMAX,IREJ)
      CALL POHIST(-1,WEIGHT)
      CALL POLUHI(-1,WEIGHT)
C
C  generation of events, flux calculation
      CALL NEWHIS(YMIN1,YMAX1,ZERO,P1,49,IHFLX1)
      CALL NEWHIS(YMIN2,YMAX2,ZERO,P1,49,IHFLX2)
      CALL NEWHIS(YMIN1,YMAX1,ZERO,P1,49,IHFLY1)
      CALL NEWHIS(YMIN2,YMAX2,ZERO,P1,49,IHFLY2)
      Q2MINL = LOG(Q2LOW1)
      Q2MAXL = LOG(Q2MAX1)
      CALL NEWHIS(Q2MINL,Q2MAXL,ZERO,P1,49,IHFLQ1)
      Q2MINL = LOG(Q2LOW2)
      Q2MAXL = LOG(Q2MAX2)
      CALL NEWHIS(Q2MINL,Q2MAXL,ZERO,P1,49,IHFLQ2)
      ECUP = 2.D0*SQRT(YMAX1*YMAX2*EE1*EE2)*1.1D0
      ECUPL = LOG(ECUP)
      ECMINL = LOG(ECMIN)
      CALL NEWHIS(ECMINL,ECUPL,ZERO,P1,49,IGGLUM)
      AY1  = ZERO
      AY2  = ZERO
      AYS1 = ZERO
      AYS2 = ZERO
      Q21MIN = 1.D30
      Q22MIN = 1.D30
      Q21MAX = 0.D0
      Q22MAX = 0.D0
      Q21AVE = 0.D0
      Q22AVE = 0.D0
      Q21AV2 = 0.D0
      Q22AV2 = 0.D0
      AN1MIN = 1.D30
      AN2MIN = 1.D30
      AN1MAX = 0.D0
      AN2MAX = 0.D0
      YY1MIN = 1.D30
      YY2MIN = 1.D30
      YY1MAX = 0.D0
      YY2MAX = 0.D0
      NITER = NEVENT
      ITRY = 0
      ITRW = 0
      DO 200 I=1,NITER
C  sample y1, y2
 150    CONTINUE
        ITRY = ITRY+1
 175    CONTINUE
          ITRW = ITRW+1
          Y1 = EXP(X1DEL*RNDM(AY1)+X1MIN)
          Y2 = EXP(X2DEL*RNDM(AY2)+X2MIN)
          Q2LOW1 = MAX(Q2MIN1,ELEM2*Y1**2/(ONE-Y1))
          Q2LOW2 = MAX(Q2MIN2,ELEM2*Y2**2/(ONE-Y2))
          Q2LOG1 = LOG(Q2MAX1/Q2LOW1)
          Q2LOG2 = LOG(Q2MAX2/Q2LOW2)
          WGH = ((ONE+(ONE-Y1)**2)*Q2LOG1
     &        -TWO*ELEM2*Y1*(ONE/Q2LOW1-ONE/Q2MAX1)*Y1)
     &         *((ONE+(ONE-Y2)**2)*Q2LOG2
     &        -TWO*ELEM2*Y2*(ONE/Q2LOW2-ONE/Q2MAX2)*Y2)
          IF(WGMAX.LT.WGH) THEN
            WRITE(6,'(1X,A,4E12.5)')
     &        'GGEPEM:WEIGHT ERROR:',Y1,Y2,WGMAX,WGH
          ENDIF
        IF(RNDM(AYS1)*WGMAX.GT.WGH) GOTO 175
C  sample Q2
        YEFF = ONE+(ONE-Y1)**2
 185    CONTINUE
          Q2P1 = Q2LOW1*EXP(Q2LOG1*RNDM(Y1))
          WEIGHT = (YEFF-2.D0*(ONE-Y1)*Q2LOW1/Q2P1)/YEFF
        IF(WEIGHT.LT.RNDM(Q2P1)) GOTO 185
C
        YEFF = ONE+(ONE-Y2)**2
 186    CONTINUE
          Q2P2 = Q2LOW2*EXP(Q2LOG2*RNDM(Y2))
          WEIGHT = (YEFF-2.D0*(ONE-Y2)*Q2LOW2/Q2P2)/YEFF
        IF(WEIGHT.LT.RNDM(Q2P2)) GOTO 186
C
        GYY(1) = Y1
        GQ2(1) = Q2P1
        GYY(2) = Y2
        GQ2(2) = Q2P2
C
        CALL FILHIS(Y1,ONE,IHFLX1)
        CALL ADDHIS(IHFLX1)
        CALL FILHIS(Y2,ONE,IHFLX2)
        CALL ADDHIS(IHFLX2)
C  incoming electron 1
        PINI(1,1) = ZERO
        PINI(2,1) = ZERO
        PINI(3,1) = EE1
        PINI(4,1) = EE1
        PINI(5,1) = ZERO
C  outgoing electron 1
        YQ2 = SQRT((ONE-Y1)*Q2P1)
        Q2E = Q2P1/(4.D0*EE1)
        E1Y = EE1*(ONE-Y1)
        CALL SFECFE(SIF1,COF1)
        PFIN(1,1) = YQ2*COF1
        PFIN(2,1) = YQ2*SIF1
        PFIN(3,1) = E1Y-Q2E
        PFIN(4,1) = E1Y+Q2E
        PFIN(5,1) = ZERO
C  photon 1
        P1(1) = -PFIN(1,1)
        P1(2) = -PFIN(2,1)
        P1(3) = PINI(3,1)-PFIN(3,1)
        P1(4) = PINI(4,1)-PFIN(4,1)
C  incoming electron 2
        PINI(1,2) = ZERO
        PINI(2,2) = ZERO
        PINI(3,2) = -EE2
        PINI(4,2) = EE2
        PINI(5,2) = ZERO
C  outgoing electron 2
        YQ2 = SQRT((ONE-Y2)*Q2P2)
        Q2E = Q2P2/(4.D0*EE2)
        E1Y = EE2*(ONE-Y2)
        CALL SFECFE(SIF2,COF2)
        PFIN(1,2) = YQ2*COF2
        PFIN(2,2) = YQ2*SIF2
        PFIN(3,2) = -E1Y+Q2E
        PFIN(4,2) = E1Y+Q2E
        PFIN(5,2) = ZERO
C  photon 2
        P2(1) = -PFIN(1,2)
        P2(2) = -PFIN(2,2)
        P2(3) = PINI(3,2)-PFIN(3,2)
        P2(4) = PINI(4,2)-PFIN(4,2)
C  ECMS cut
        GGECM = (P1(4)+P2(4))**2-(P1(1)+P2(1))**2
     &         -(P1(2)+P2(2))**2-(P1(3)+P2(3))**2
        IF(GGECM.LT.0.1D0) GOTO 175
        GGECM = SQRT(GGECM)
        IF((GGECM.LT.ECMIN).OR.(GGECM.GT.ECMAX)) GOTO 175
C  electron tagger 1
        PFPHI(1) = ATAN2(COF1,SIF1)
        PFTHE(1) = ACOS(PFIN(3,1)/PFIN(4,1))
        IF(PFIN(4,1).GT.EEMIN1) THEN
          IF((PFTHE(1).LT.THMIN1).OR.(PFTHE(1).GT.THMAX1)) GOTO 175
        ENDIF
C  electron tagger 2
        PFPHI(2) = ATAN2(COF2,SIF2)
        PFTHE(2) = ACOS(PFIN(3,2)/PFIN(4,2))
        IF(PFIN(4,2).GT.EEMIN2) THEN
          IF((PFTHE(2).LT.THMIN2).OR.(PFTHE(2).GT.THMAX2)) GOTO 175
        ENDIF
C
        PGAM(1,1) = P1(1)
        PGAM(2,1) = P1(2)
        PGAM(3,1) = P1(3)
        PGAM(4,1) = P1(4)
        PGAM(5,1) = -SQRT(Q2P1)
        PGAM(1,2) = P2(1)
        PGAM(2,2) = P2(2)
        PGAM(3,2) = P2(3)
        PGAM(4,2) = P2(4)
        PGAM(5,2) = -SQRT(Q2P2)
        CALL PRESEL(5,IREJ)
        IF(IREJ.NE.0) GOTO 175
C  event generation
        CALL EVENT(1,P1,P2,SIGCUR,IREJ)
        IF(IREJ.NE.0) GOTO 150
C  internal histograms
	GGECML = LOG(GGECM)
        CALL FILHIS(GGECML,ONE,IGGLUM)
        CALL ADDHIS(IGGLUM)
        CALL FILHIS(Y1,ONE,IHFLY1)
        CALL ADDHIS(IHFLY1)
        CALL FILHIS(Y2,ONE,IHFLY2)
        CALL ADDHIS(IHFLY2)
        Q2L = LOG(Q2P1)
        CALL FILHIS(Q2L,ONE,IHFLQ1)
        CALL ADDHIS(IHFLQ1)
        Q2L = LOG(Q2P2)
        CALL FILHIS(Q2L,ONE,IHFLQ2)
        CALL ADDHIS(IHFLQ2)
C  statistics
        AY1  = AY1+Y1
        AYS1 = AYS1+Y1*Y1
        AY2  = AY2+Y2
        AYS2 = AYS2+Y2*Y2
        Q21MIN = MIN(Q21MIN,Q2P1)
        Q22MIN = MIN(Q22MIN,Q2P2)
        Q21MAX = MAX(Q21MAX,Q2P1)
        Q22MAX = MAX(Q22MAX,Q2P2)
        AN1MIN = MIN(AN1MIN,PFTHE(1))
        AN2MIN = MIN(AN2MIN,PFTHE(2))
        AN1MAX = MAX(AN1MAX,PFTHE(1))
        AN2MAX = MAX(AN2MAX,PFTHE(2))
        YY1MIN = MIN(YY1MIN,Y1)
        YY2MIN = MIN(YY2MIN,Y2)
        YY1MAX = MAX(YY1MAX,Y1)
        YY2MAX = MAX(YY2MAX,Y2)
        Q21AVE = Q21AVE+Q2P1
        Q22AVE = Q22AVE+Q2P2
        Q21AV2 = Q21AV2+Q2P1*Q2P1
        Q22AV2 = Q22AV2+Q2P2*Q2P2
C  histograms
        CALL POHIST(1,ONE)
        CALL POLUHI(1,ONE)
 200  CONTINUE
C
      WGY  = WGMAX*DBLE(ITRY)/DBLE(ITRW)/(137.D0*TWO*PI)**2
      WGY  = WGY*LOG(YMAX1/YMIN1)*LOG(YMAX2/YMIN2)
      AY1  = AY1/DBLE(NITER)
      AYS1 = AYS1/DBLE(NITER)
      DAY1 = SQRT((AYS1-AY1**2)/DBLE(NITER))
      AY2  = AY2/DBLE(NITER)
      AYS2 = AYS2/DBLE(NITER)
      DAY2 = SQRT((AYS2-AY2**2)/DBLE(NITER))
      Q21AVE = Q21AVE/DBLE(NITER)
      Q21AV2 = Q21AV2/DBLE(NITER)
      Q21AV2 = SQRT((Q21AV2-Q21AVE**2)/DBLE(NITER))
      Q22AVE = Q22AVE/DBLE(NITER)
      Q22AV2 = Q22AV2/DBLE(NITER)
      Q22AV2 = SQRT((Q22AV2-Q22AVE**2)/DBLE(NITER))
      WEIGHT = WGY*SIGMAX*DBLE(NITER)/DBLE(ITRY)
C  output of statistics, histograms
      WRITE(6,'(//1X,A,/1X,A,1PE12.3,A,/1X,A)')
     &'=========================================================',
     &' *****   simulated cross section: ',WEIGHT,' mb  *****',
     &'========================================================='
      WRITE(6,'(//1X,A,3I10)')
     &  'GGEPEM:SUMMARY:NITER,ITRY,ITRW',NITER,ITRY,ITRW
      WRITE(6,'(1X,A,1P2E12.4)') 'EFFECTIVE WEIGHT (FLUX,TOTAL)',
     &  WGY,WEIGHT
      WRITE(6,'(1X,A,1P2E12.4)') 'AVERAGE Y1,DY1               ',
     &  AY1,DAY1
      WRITE(6,'(1X,A,1P2E12.4)') 'AVERAGE Y2,DY2               ',
     &  AY2,DAY2
      WRITE(6,'(1X,A,1P2E12.4)') 'SAMPLED Y RANGE PHOTON 1     ',
     &  YY1MIN,YY1MAX
      WRITE(6,'(1X,A,1P2E12.4)') 'SAMPLED Y RANGE PHOTON 2     ',
     &  YY2MIN,YY2MAX
      WRITE(6,'(1X,A,1P2E12.4)') 'AVERAGE Q2,DQ2 PHOTON 1      ',
     &  Q21AVE,Q21AV2
      WRITE(6,'(1X,A,1P2E12.4)') 'SAMPLED Q2 RANGE PHOTON 1    ',
     &  Q21MIN,Q21MAX
      WRITE(6,'(1X,A,1P2E12.4)') 'AVERAGE Q2,DQ2  PHOTON 2     ',
     &  Q22AVE,Q22AV2
      WRITE(6,'(1X,A,1P2E12.4)') 'SAMPLED Q2 RANGE PHOTON 2    ',
     &  Q22MIN,Q22MAX
      WRITE(6,'(1X,A,1P2E12.4)') 'SAMPLED THETA RANGE ELECTRON1',
     &  AN1MIN,AN1MAX
      WRITE(6,'(1X,A,1P4E12.4)') 'SAMPLED THETA RANGE ELECTRON2',
     &  AN2MIN,AN2MAX,PI-AN2MAX,PI-AN2MIN
C
      CALL EVENT(-2,P1,P2,WEIGHT,IREJ)
      IF(NITER.GT.1) THEN
        CALL POHIST(-2,WEIGHT)
        CALL POLUHI(-2,WEIGHT)
      ELSE
        WRITE(6,'(1X,A,I4)') 'GGEPEM:NO OUTPUT OF HISTOGRAMS',NITER
      ENDIF
C
      FAC =  WGY
      TITLE = ' BREMSSTRAHLUNG PHOTON-PHOTON LUMINOSITY DPHI/D(ECM-GG)'
      CALL OUTHIS(IGGLUM,3,TITLE,FAC,1)
      FAC = SQRT(WGY)
      TITLE = ' EP APPROXIMATION PHOTON FLUX SIDE 1'
      CALL OUTHIS(IHFLX1,1,TITLE,FAC,1)
      TITLE = ' EP APPROXIMATION PHOTON FLUX SIDE 2'
      CALL OUTHIS(IHFLX2,1,TITLE,FAC,1)
      TITLE = ' EFFECTIVE PHOTON FLUX SIDE 1 (CROSS SECTION WEIGHTED)'
      CALL OUTHIS(IHFLY1,1,TITLE,FAC,1)
      TITLE = ' EFFECTIVE PHOTON FLUX SIDE 2 (CROSS SECTION WEIGHTED)'
      CALL OUTHIS(IHFLY2,1,TITLE,FAC,1)
      TITLE = ' BREMSSTRAHLUNG PHOTON VIRTUALITY 1 (GeV**2)'
      CALL OUTHIS(IHFLQ1,3,TITLE,FAC,1)
      TITLE = ' BREMSSTRAHLUNG PHOTON VIRTUALITY 2 (GeV**2)'
      CALL OUTHIS(IHFLQ2,3,TITLE,FAC,1)
      END
