C
C
      SUBROUTINE PARTPT(IMODE,IF,IL,PTCUT,IREJ)
C********************************************************************
C
C    assign to parton level particles soft/hard pt
C    and append these partons on COMMON /HEPEVS/
C
C    input:  IMODE   -2   output of statistics
C                    -1   initialization
C                     0   sampling of pt for soft partons belonging to
C                         soft Pomerons
C                     1   sampling of pt for soft partons belonging to
C                         hard Pomerons
C            IF           first entry in /HEPEVE/ to check
C            IL           last entry in /HEPEVE/ to check
C            PTCUT        current value of PTCUT between soft and hard
C
C    output: IREJ     0   success
C                     1   failure
C
C    (soft pt is sampled by call to SOFTPT)
C
C********************************************************************
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      SAVE
C
      PARAMETER (ZERO   =  0.D0,
     &           ONE    =  1.D0,
     &           TWO    =  2.D0,
     &           THREE  =  3.D0,
     &           OHALF  =  0.5D0,
     &           PI2    =  6.28318530718D0,
     &           DEPS   =  1.D-40)
      PARAMETER (NMAXD=100)
      COMMON /DEBUG/ IDEB(NMAXD),KSPOM,KHPOM,KSREG,KHDIR,KACCEP,
     &  KSTRG,KHTRG,KSLOO,KHLOO,KSDPO,KHDPO,KEVENT,KSOFT,KHARD
C
      CHARACTER*8 MDLNA
      COMMON /MODELS/ MDLNA(50),ISWMDL(50),PARMDL(200),IPAMDL(200)
      COMMON /GLOCMS/ ECM,PCM,PMASS(2),PVIRT(2),IFPAP(2),IFPAB(2)
C
      PARAMETER (NMXHEP=2000)
      COMMON/HEPEVS/NEVHEP,NHEP,ISTHEP(NMXHEP),IDHEP(NMXHEP),
     &JMOHEP(2,NMXHEP),JDAHEP(2,NMXHEP),PHEP(5,NMXHEP),VHEP(4,NMXHEP)
      COMMON/HEPEVE/IMPART(NMXHEP),IPHIST(2,NMXHEP),ICOLOR(2,NMXHEP)
C
      DIMENSION PTS(2,100),XP(100),MODIFY(100),P(2,2),IV(100)
C
C  debug output
      IF(IDEB(6).GE.10) WRITE(6,'(1X,A,3I4)')
     &  'PARTPT:DEBUG:MODE,FIRST,LAST',IMODE,IF,IL
C
      IF(IMODE.LT.0) GOTO 1000
C
      IREJ = 0
      IF((ISWMDL(3).EQ.10).AND.(ISWMDL(4).EQ.10)) RETURN
C
C  find first parton
      DO 100 ISTART=IF,IL
        IF(ISTHEP(ISTART).EQ.-1) GOTO 200
 100  CONTINUE
 200  CONTINUE
C  count entries to modify
      IENTRY = 0
      PTCUT2 = PTCUT**2
      EMIN = 1.D20
      IPEAK = 1
C  soft Pomerons
      IF(IMODE.EQ.0) THEN
        DO 300 I=ISTART,IL
          IF((ISTHEP(I).EQ.-1).AND.(ABS(IPHIST(1,I)).LT.100)) THEN
            IF((PHEP(1,I)**2+PHEP(2,I)**2).LT.PTCUT2) THEN
              IENTRY = IENTRY+1
              MODIFY(IENTRY) = I
              XP(IENTRY) = SIGN(PHEP(4,I)/ECM,PHEP(3,I))
              IV(IENTRY) = ICOLOR(2,I)
              IF(PHEP(4,I).LT.EMIN) THEN
                EMIN = PHEP(4,I)
                IPEAK = IENTRY
              ENDIF
            ENDIF
          ENDIF
 300    CONTINUE
C  hard Pomerons
      ELSE IF(IMODE.EQ.1) THEN
        DO 350 I=ISTART,IL
          IF((ISTHEP(I).EQ.-1).AND.(IPHIST(1,I).GE.100)) THEN
            IF((PHEP(1,I)**2+PHEP(2,I)**2).LT.PTCUT2) THEN
              IENTRY = IENTRY+1
              MODIFY(IENTRY) = I
              XP(IENTRY) = SIGN(PHEP(4,I)/ECM,PHEP(3,I))
              IF(ISWMDL(24).EQ.0) THEN
                IV(IENTRY) = ICOLOR(2,I)
              ELSE IF(ISWMDL(24).EQ.1) THEN
                IV(IENTRY) = -1
              ELSE
                IV(IENTRY) = 1
              ENDIF
              IF(PHEP(4,I).LT.EMIN) THEN
                EMIN = PHEP(4,I)
                IPEAK = IENTRY
              ENDIF
            ENDIF
          ENDIF
 350    CONTINUE
      ELSE
        WRITE(6,'(/1X,A,I5)') 'PARTPT:ERROR:INVALID MODE',IMODE
        CALL POABRT
      ENDIF
C  debug output
      IF(IDEB(6).GE.5) THEN
        WRITE(6,'(1X,A,3I4)')
     &    'PARTPT:DEBUG:NUMBER OF PARTONS,IPEAK,MODE',IENTRY,IPEAK,IMODE
        IF(IDEB(6).GE.20) CALL POPREV(0)
      ENDIF
      IF(IENTRY.LE.1) RETURN
C
C  sample pt of soft partons
      IF(ISWMDL(5).LE.1) THEN
C  energy limited sampling
        CALL SWAPI(MODIFY(IPEAK),MODIFY(1))
        ITER = 0
 400    CONTINUE
          PSUMX = ZERO
          PSUMY = ZERO
          ITER = ITER+1
          IF(ITER.GE.1000) THEN
            WRITE(6,'(/1X,A,2I5)') 'PARTPT:WARNING:REJECT IMOD,ITER',
     &        IENTRY,ITER
            IREJ = 1
            RETURN
          ENDIF
          DO 410 I=2,IENTRY
            II = MODIFY(I)
            PTMX = MIN(PHEP(4,II),PTCUT)
            IF(ISWMDL(5).EQ.0) THEN
              CALL SOFTPT(1,PTCUT,PTMX,XP(II),IV(II),P)
            ELSE
              CALL SOFTPT(1,PTMX,PTMX,XP(II),IV(II),P)
            ENDIF
            PTS(1,I) = P(1,1)
            PTS(2,I) = P(2,1)
            PSUMX = PSUMX+P(1,1)
            PSUMY = PSUMY+P(2,1)
 410      CONTINUE
          PTREM = SQRT(PSUMX**2+PSUMY**2)
        IF(PTREM.GT.MIN(PHEP(4,I),PTCUT)) GOTO 400
        PTS(1,1) = -PSUMX
        PTS(2,1) = -PSUMY
      ELSE IF(ISWMDL(5).EQ.2) THEN
C  unlimited sampling
        IPEAK = RNDM(PSUMX)*IENTRY+1
        CALL SWAPI(MODIFY(IPEAK),MODIFY(1))
        CALL SWAPD(XP(IPEAK),XP(1))
        CALL SWAPI(IV(IPEAK),IV(1))
        CALL SOFTPT(IENTRY,PTCUT,PTCUT,XP,IV,PTS)
      ELSE
        WRITE(6,'(/1X,A,I4)') 'PARTPT:UNSUPPORTED ISWMDL(5):',ISWMDL(5)
        CALL POABRT
      ENDIF
C  change parton in COMMON HEPEVS
      DO 500 II=1,IENTRY
        I = MODIFY(II)
        PHEP(1,I) = PHEP(1,I)+PTS(1,II)
        PHEP(2,I) = PHEP(2,I)+PTS(2,II)
        AMSQR = PHEP(4,I)**2
     &           -PHEP(1,I)**2-PHEP(2,I)**2-PHEP(3,I)**2
        PHEP(5,I) = SIGN(SQRT(ABS(AMSQR)),AMSQR)
 500  CONTINUE
C  debug output
      IF(IDEB(6).GE.15) THEN
        WRITE(6,'(1X,A,/1X,A)') 'PARTPT:DEBUG OUTPUT:',
     &                        '===================='
        CALL POPREV(0)
      ENDIF
      RETURN
C
C  initialization / output of statistics
 1000 CONTINUE
      CALL SOFTPT(IMODE,PTM,PTM,XP,IV,PTS)
      END
