C
C
      SUBROUTINE POHWGI(ECMX,PTCUT,NKON,Z,FDIS)
C**********************************************************************
C
C     auxiliary subroutine to find maximum of remaining weight
C
C     input:  ECMX   current CMS energy
C             PTCUT  current pt cutoff
C             NKON   process label  1..5  resolved
C                                   6..7  direct particle 1
C                                   8..9  direct particle 2
C                                   10    double direct
C             Z(3)   transformed variable
C
C     output: remaining weight
C
C**********************************************************************
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      SAVE
C
      DIMENSION Z(3)
C
      PARAMETER ( NKM   = 10 )
      PARAMETER ( TINY  = 1.D-30,
     &            ONE   = 1.D0,
     &            TINY6 = 1.D-06,
     &            ZERO  = 0.D0 )
      PARAMETER (NMAXD=100)
C
      COMMON /DEBUG/ IDEB(NMAXD),KSPOM,KHPOM,KSREG,KHDIR,KACCEP,
     &  KSTRG,KHTRG,KSLOO,KHLOO,KSDPO,KHDPO,KEVENT,KSOFT,KHARD
      CHARACTER*8 MDLNA
      COMMON /MODELS/ MDLNA(50),ISWMDL(50),PARMDL(200),IPAMDL(200)
C  CM system of Pomeron/Reggeon exchange
      COMMON /POMCMS/ ECMP,PCMP,PMASSP(2),PVIRTP(2),GAMBEP(4),
     &                SIDP,CODP,SIFP,COFP,NPOSP(2),
     &                IDPDG1,IDBAM1,IDPDG2,IDBAM2
      CHARACTER*8 PDFNAM
      COMMON /PARPDF/ PDFLAM(2),PDFQ2M(2),PDFNAM(2),IPARID(2),PARVI(2),
     &                IPAVA(2),ITYPE(2),IGRP(2),ISET(2),IEXT(2),NPAOLD
      COMMON /HAPARA/ ALPHAS,Q0SQR,ALASQR,BQCD,NPD,NF,NHA,NHB
      COMMON /HADRON/ QMASS(6),BET,PCOUDI,PNORM,VALPRG(2),NFS
      COMMON /HAQQAP/ AQQAL,AQQALI,AQQALF,AQQPD,
     &                NQQAL,NQQALI,NQQALF,NQQPD
      COMMON /HASCA / PTWANT,AS,AH,ALNS,ALNH,Z1MAX,Z1DIF,Z2MAX,Z2DIF,
     &                PT,ETAC,ETAD,X1,X2,V,U,QQPD,QQAL,
     &                W,W1,AXX,WEIGHT,MSPR,IREJSC
      DIMENSION F(NKM),PDA(-6:6),PDB(-6:6)
**sr 29.04. commented to use dp-JETSET
      REAL ULALEM,SSR,QQALR
**
C
      FDIS = ZERO
C
      IF(IDEB(64).GE.25) WRITE(6,'(1X,A,/5X,5E12.3,I5)')
     &  'POHWGI:DEBUG:ECM,PT,Z,NK',ECMX,PTCUT,Z,NKON
C  check input values
      IF ( Z(1).LT.0.D0  .OR.  Z(1).GT.1.D0 ) RETURN
      IF ( Z(2).LT.0.D0  .OR.  Z(2).GT.1.D0 ) RETURN
      IF ( Z(3).LT.0.D0  .OR.  Z(3).GT.1.D0 ) RETURN
C  transformations
      Y1    = EXP(ALNH*Z(1))
      IF(NKON.LE.5) THEN
C  resolved kinematic
        Y2  =-(1.D0-Y1)+2.D0*(1.D0-Y1)*Z(2)
        X1  = 0.5D0*(Y2+SQRT(Y2*Y2+4.D0*Y1))
        X2  = X1-Y2
        X1 = MIN(X1,0.999999999999D0)
        X2 = MIN(X2,0.999999999999D0)
      ELSE IF(NKON.LE.7) THEN
C  direct kinematic 1
        X1 = ONE
        X2 = MIN(Y1,0.999999999999D0)
      ELSE IF(NKON.LE.9) THEN
C  direct kinematic 2
        X1 = MIN(Y1,0.999999999999D0)
        X2 = ONE
      ELSE
C  double direct kinematic
        X1 = ONE
        X2 = ONE
      ENDIF
      W   = SQRT(MAX(TINY,1.D0-AH/Y1))
      V   =-0.5D0+W*(Z(3)-0.5D0)
      U   =-(1.D0+V)
      PT  = MAX(SQRT(U*V*Y1*ECMX*ECMX),PTCUT)
C
C  set hard scale  QQ  for alpha and partondistr.
      IF     ( NQQAL.EQ.1 ) THEN
        QQAL = AQQAL*PT*PT
      ELSEIF ( NQQAL.EQ.2 ) THEN
        QQAL = AQQAL*Y1*ECMX*ECMX*U*V/(1.+V*V+U*U)
      ELSEIF ( NQQAL.EQ.3 ) THEN
        QQAL = AQQAL*Y1*ECMX*ECMX
      ELSEIF ( NQQAL.EQ.4 ) THEN
        QQAL = AQQAL*Y1*ECMX*ECMX*(U*V)**(1./3.)
      ENDIF
      IF     ( NQQPD.EQ.1 ) THEN
        QQPD = AQQPD*PT*PT
      ELSEIF ( NQQPD.EQ.2 ) THEN
        QQPD = AQQPD*Y1*ECMX*ECMX*U*V/(1.+V*V+U*U)
      ELSEIF ( NQQPD.EQ.3 ) THEN
        QQPD = AQQPD*Y1*ECMX*ECMX
      ELSEIF ( NQQPD.EQ.4 ) THEN
        QQPD = AQQPD*Y1*ECMX*ECMX*(U*V)**(1./3.)
      ENDIF
C
      IF(NKON.LE.5) THEN
        DO 10 N=1,5
          F(N) = ZERO
 10     CONTINUE
C  resolved processes
        ALPHA1 = (BQCD/LOG(MAX(QQAL/(PDFLAM(1)*PDFLAM(2)),1.1D0)))
        ALPHA2 = ALPHA1
        CALL PDIS(X1,QQPD,1,PDA)
        CALL PDIS(X2,QQPD,2,PDB)
C  calculate full distribution FDIS
        DO 20 I=1,NF
          F(2) = F(2)+PDA(I)*PDB(-I)+PDA(-I)*PDB( I)
          F(3) = F(3)+PDA(I)*PDB( I)+PDA(-I)*PDB(-I)
          F(4) = F(4)+PDA(I)+PDA(-I)
          F(5) = F(5)+PDB(I)+PDB(-I)
20      CONTINUE
        F(1)   = PDA(0)*PDB(0)
        T      = PDA(0)*F(5)+PDB(0)*F(4)
        F(5)   = F(4)*F(5)-(F(2)+F(3))
        F(4)   = T
      ELSE IF(NKON.LE.7) THEN
C  direct processes particle 1
        IF(IDPDG1.EQ.22) THEN
          QQALR = QQAL
          ALPHA1 = DBLE(ULALEM(QQALR))
*         ALPHA1 = 1.D0/137.D0
          CH1 = 4.D0/9.D0
          CH2 = 3.D0/9.D0
        ELSE IF(IDPDG1.EQ.45) THEN
          ALPHA1 = PARMDL(74)
          CH1 = ONE
          CH2 = ZERO
        ELSE
          FDIS = -1.D0
          RETURN
        ENDIF
        ALPHA2 = BQCD/LOG(MAX(QQAL/PDFLAM(2)**2,1.1D0))
        CALL PDIS(X2,QQPD,2,PDB)
        F(6) = ZERO
        DO 30 I=1,NF
          F(6) = F(6)+(PDB(I)+PDB(-I))*(CH1-CH2*MOD(I,2))
 30     CONTINUE
        F(7)   = PDB(0)
      ELSE IF(NKON.LE.9) THEN
C  direct processes particle 2
        ALPHA1 = BQCD/LOG(MAX(QQAL/PDFLAM(1)**2,1.1*ONE))
        IF(IDPDG2.EQ.22) THEN
          QQALR = QQAL
          ALPHA2 = DBLE(ULALEM(QQALR))
*         ALPHA2 = 1.D0/137.D0
          CH1 = 4.D0/9.D0
          CH2 = 3.D0/9.D0
        ELSE IF(IDPDG2.EQ.45) THEN
          ALPHA2 = PARMDL(74)
          CH1 = ONE
          CH2 = ZERO
        ELSE
          FDIS = -1.D0
          RETURN
        ENDIF
        CALL PDIS(X1,QQPD,1,PDA)
        F(8) = ZERO
        DO 40 I=1,NF
          F(8) = F(8)+(PDA(I)+PDA(-I))*(CH1-CH2*MOD(I,2))
 40     CONTINUE
        F(9)   = PDA(0)
      ELSE
C  double direct process
        SSR = ECMX*ECMX
        IF(IDPDG1.EQ.22) THEN
          ALPHA1 = DBLE(ULALEM(SSR))
*         ALPHA1 = 1.D0/137.D0
        ELSE IF(IDPDG1.EQ.45) THEN
          ALPHA1 = PARMDL(74)
        ELSE
          FDIS = -1.D0
          RETURN
        ENDIF
        IF(IDPDG2.EQ.22) THEN
          ALPHA1 = DBLE(ULALEM(SSR))
*         ALPHA2 = 1.D0/137.D0
        ELSE IF(IDPDG2.EQ.45) THEN
          ALPHA2 = PARMDL(74)
        ELSE
          FDIS = -1.D0
          RETURN
        ENDIF
        F(10) = ONE
      ENDIF
C
      FDIS   = MAX(ZERO,F(NKON)*ALPHA1*ALPHA2)
C  debug output
      IF(IDEB(64).GE.20) WRITE(6,'(1X,A,/2X,I3,2I6,7E11.3)')
     &  'POHWGI:DEBUG:NKON,ID1,ID2,AL1,AL2,X1,X2,PT,F(NKON),FDIS',
     &  NKON,IDPDG1,IDPDG2,ALPHA1,ALPHA2,X1,X2,PT,F(NKON),FDIS
      END
