C
C
      SUBROUTINE POQCHK(MODE,IDEV)
C**********************************************************************
C
C     check quantum numbers of entries in COMMON HEPEVS
C           ( energy, momentum, charge, baryon number conservation)
C
C     input:    MODE    -1  check overall momentum conservation
C                           and perform detailed check in case of
C                           deviations
C                        1  test all branchings, mother-daughter
C                           relations
C
C     output:   IDEV     0  no deviations
C                        1  deviations found
C
C**********************************************************************
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      SAVE
C
      PARAMETER (NMAXD=100)
      COMMON /DEBUG/ IDEB(NMAXD),KSPOM,KHPOM,KSREG,KHDIR,KACCEP,
     &  KSTRG,KHTRG,KSLOO,KHLOO,KSDPO,KHDPO,KEVENT,KSOFT,KHARD
C
      PARAMETER (ZERO   =  0.D0,
     &           ONE    =  1.D0,
     &           TWO    =  2.D0,
     &           THREE  =  3.D0,
     &           OHALF  =  0.5D0)
C
      CHARACTER*8 MDLNA
      COMMON /MODELS/ MDLNA(50),ISWMDL(50),PARMDL(200),IPAMDL(200)
C  global CMS energy
      COMMON /GLOCMS/ ECM,PCM,PMASS(2),PVIRT(2),IFPAP(2),IFPAB(2)
C
      PARAMETER (NMXHEP=2000)
      COMMON/HEPEVS/NEVHEP,NHEP,ISTHEP(NMXHEP),IDHEP(NMXHEP),
     &JMOHEP(2,NMXHEP),JDAHEP(2,NMXHEP),PHEP(5,NMXHEP),VHEP(4,NMXHEP)
      COMMON/HEPEVE/IMPART(NMXHEP),IPHIST(2,NMXHEP),ICOLOR(2,NMXHEP)
C
      PARAMETER (MSTR=100)
      COMMON /STRING/ GAMBET(4,MSTR),NPOS(3,MSTR),NCODE(MSTR),
     &     IPAR1(MSTR),IPAR2(MSTR),IPAR3(MSTR),IPAR4(MSTR),
     &     NNCH(MSTR),IBHAD(MSTR),ISTR
C
C  count number of errors to avoid disk overflow
      DATA IERR / 0 /
C
      IDEV = 0
C  conservation check suppressed
      IF((IPAMDL(15).EQ.0).OR.(IDEB(20).LE.-10)) RETURN
C  dtunuc call
      IF(IPAMDL(13).GT.0) RETURN
C
      DDREL = PARMDL(75)
      DDABS = PARMDL(76)
      IF(MODE.EQ.-1) GOTO 500
C
 50   CONTINUE
C
      I = 1
 100  CONTINUE
C  recognize only decayed particles as mothers
        IF(ISTHEP(I).EQ.2) THEN
C  search for other mother particles
          K = JDAHEP(1,I)
          K1 = JMOHEP(1,K)
          K2 = JMOHEP(2,K)
C  sum over mother particles
          ICH1 = IPOCH3(K1,2)
          IBA1 = IPOBA3(K1,2)
          EE1 = PHEP(4,K1)
          PX1 = PHEP(1,K1)
          PY1 = PHEP(2,K1)
          PZ1 = PHEP(3,K1)
          IF(K2.LT.0) THEN
            K2 = -K2
            IF((K1.GT.I).OR.(K2.LT.I)) THEN
              WRITE(6,'(/,1X,A,3I4)')
     &        'POQCHK:WARNING:INCONSISTENT MOTHER/DAUGHTER RELATION ',
     &        I,K1,K2
              CALL POPREV(-1)
            ENDIF
            DO 400 II=K1+1,K2
              IF(ABS(ISTHEP(II)).LE.2) THEN
                ICH1 = ICH1 + IPOCH3(II,2)
                IBA1 = IBA1 + IPOBA3(II,2)
                EE1 = EE1 + PHEP(4,II)
                PX1 = PX1 + PHEP(1,II)
                PY1 = PY1 + PHEP(2,II)
                PZ1 = PZ1 + PHEP(3,II)
              ENDIF
 400        CONTINUE
          ELSE IF((K2.GT.0).AND.(K2.NE.K1)) THEN
            ICH1 = ICH1 + IPOCH3(K2,2)
            IBA1 = IBA1 + IPOBA3(K2,2)
            EE1 = EE1 + PHEP(4,K2)
            PX1 = PX1 + PHEP(1,K2)
            PY1 = PY1 + PHEP(2,K2)
            PZ1 = PZ1 + PHEP(3,K2)
          ENDIF
C  sum over daughter particles
          ICH2 = ZERO
          IBA2 = ZERO
          EE2 = ZERO
          PX2 = ZERO
          PY2 = ZERO
          PZ2 = ZERO
          DO 200 II=JDAHEP(1,I),JDAHEP(2,I)
            IF(ABS(ISTHEP(II)).LE.2) THEN
              ICH2 = ICH2 + IPOCH3(II,2)
              IBA2 = IBA2 + IPOBA3(II,2)
              EE2 = EE2 + PHEP(4,II)
              PX2 = PX2 + PHEP(1,II)
              PY2 = PY2 + PHEP(2,II)
              PZ2 = PZ2 + PHEP(3,II)
            ENDIF
 200      CONTINUE
C  conservation check
          ESC = MAX(MAX(EE1,EE2)*DDREL,DDABS)
          IF(ABS(EE1-EE2).GT.ESC) THEN
            WRITE(6,'(1X,A,I3,2X,2E12.4)')
     &        'POQCHK:WARNING:ENERGY TEST PARTICLE NO ',I,EE1,EE2
            IDEV = 1
          ENDIF
          ESC = MAX(MAX(ABS(PX1),ABS(PX2))*DDREL,DDABS)
          IF(ABS(PX1-PX2).GT.ESC) THEN
            WRITE(6,'(1X,A,I3,2X,2E12.4)')
     &        'POQCHK:WARNING:MOMENTUM X TEST PARTICLE NO ',I,PX1,PX2
            IDEV = 1
          ENDIF
          ESC = MAX(MAX(ABS(PY1),ABS(PY2))*DDREL,DDABS)
          IF(ABS(PY1-PY2).GT.ESC) THEN
            WRITE(6,'(1X,A,I3,2X,2E12.4)')
     &        'POQCHK:WARNING:MOMENTUM Y TEST PARTICLE NO ',I,
     &        PY1,PY2
            IDEV = 1
          ENDIF
          ESC = MAX(MAX(ABS(PZ1),ABS(PZ2))*DDREL,DDABS)
          IF(ABS(PZ1-PZ2).GT.ESC) THEN
            WRITE(6,'(1X,A,I3,2X,2E12.3)')
     &        'POQCHK:WARNING:MOMENTUM Z TEST PARTICLE NO ',I,
     &        PZ1,PZ2
            IDEV = 1
          ENDIF
          IF(ICH1.NE.ICH2) THEN
            WRITE(6,'(1X,A,I3,2X,2I5)')
     &      'POQCHK:WARNING:ELE. CHARGE CONSERVATION PARTICLE NO ',I,
     &      ICH1,ICH2
            IDEV = 1
          ENDIF
          IF(IBA1.NE.IBA2) THEN
            WRITE(6,'(1X,A,I3,2X,2I5)')
     &      'POQCHK:WARNING:BAR. CHARGE CONSERVATION PARTICLE NO ',I,
     &      IBA1,IBA2
            IDEV = 1
          ENDIF
          IF(IDEB(20).GE.35) THEN
            WRITE(6,
     &        '(/,1X,A,A,2(2X,I4,A,I4),2(/,5X,A,4E13.4),/5X,A,4I5)')
     &      'POQCHK DIAGNOSTICS:',
     &      '(1.MOTHER/L.MOTHER,1.DAUGHTER/L.DAUGHTER):',
     &      K1,'/',K2,JDAHEP(1,I),'/',JDAHEP(2,I),
     &      'MOTHER MOMENTA   ',PX1,PY1,PZ1,EE1,
     &      'DAUGHTER MOMENTA ',PX2,PY2,PZ2,EE2,
     &      'CHARGE,BARYON NO ',ICH1,ICH2,IBA1,IBA2
          ENDIF
        ENDIF
        I = I+1
      IF(I.LE.NHEP) GOTO 100
C
      IERR = IERR+IDEV
C
C  write complete event in case of deviations
      IF((IDEB(20).GE.0).AND.(IDEV.NE.0)) THEN
        CALL POPREV(0)
        IF(ISTR.GT.0) THEN
          CALL POPRST
          IF(ISWMDL(6).GE.0) CALL LULIST(1)
        ENDIF
      ENDIF
C
C  stop after too many errors
      IF(IERR.GT.IPAMDL(179)) THEN
        WRITE(6,'(/1X,A,I6)')
     &    'POQCHK:ERROR:TOO MANY INCONSISTENCIES',IERR
        CALL POABRT
      ENDIF
C
      RETURN
C
C  overall check only (less time consuming)
C
 500  CONTINUE
      ICH1 = IPOCH3(1,2) + IPOCH3(2,2)
      IBA1 = IPOBA3(1,2) + IPOBA3(2,2)
      EE1 = PHEP(4,1) + PHEP(4,2)
      PX1 = PHEP(1,1) + PHEP(1,2)
      PY1 = PHEP(2,1) + PHEP(2,2)
      PZ1 = PHEP(3,1) + PHEP(3,2)
      ICH2 = ZERO
      IBA2 = ZERO
      EE2 = ZERO
      PX2 = ZERO
      PY2 = ZERO
      PZ2 = ZERO
C
      DO 300 K=3,NHEP
C  recognize only existing particles as possible daughters
        IF(ABS(ISTHEP(K)).EQ.1) THEN
          ICH2 = ICH2 + IPOCH3(K,2)
          IBA2 = IBA2 + IPOBA3(K,2)
          EE2 = EE2 + PHEP(4,K)
          PX2 = PX2 + PHEP(1,K)
          PY2 = PY2 + PHEP(2,K)
          PZ2 = PZ2 + PHEP(3,K)
        ENDIF
 300  CONTINUE
C  comparison
      ESC = ECM*DDREL
      IF(ABS(EE1-EE2).GT.ESC) THEN
        WRITE(6,'(1X,A,2X,2E15.5)')
     &    'POQCHK:WARNING: TOTAL ENERGY TEST',EE1,EE2
        IDEV = 1
      ENDIF
      IF(ABS(PX1-PX2).GT.ESC) THEN
        WRITE(6,'(1X,A,2X,2E15.5)')
     &    'POQCHK:WARNING: TOTAL MOMENTUM X TEST',PX1,PX2
        IDEV = 1
      ENDIF
      IF(ABS(PY1-PY2).GT.ESC) THEN
        WRITE(6,'(1X,A,2X,2E15.5)')
     &    'POQCHK:WARNING: TOTAL MOMENTUM Y TEST',PY1,PY2
        IDEV = 1
      ENDIF
      IF(ABS(PZ1-PZ2).GT.ESC) THEN
        WRITE(6,'(1X,A,2X,2E15.5)')
     &    'POQCHK:WARNING: TOTAL MOMENTUM Z TEST',PZ1,PZ2
        IDEV = 1
      ENDIF
      IF(ICH1.NE.ICH2) THEN
        WRITE(6,'(1X,A,2X,2I5)')
     &  'POQCHK:WARNING: ELE. CHARGE CONSERVATION',ICH1,ICH2
        IDEV = 1
      ENDIF
      IF(IBA1.NE.IBA2) THEN
        WRITE(6,'(1X,A,2X,2I5)')
     &  'POQCHK:WARNING: BAR. CHARGE CONSERVATION ',IBA1,IBA2
        IDEV = 1
      ENDIF
C
C  write complete event in case of deviations
      IF((IDEB(20).GE.0).AND.(IDEV.NE.0)) THEN
        DDREL = DDREL/2.D0
        DDABS = DDABS/2.D0
        WRITE(6,'(/1X,A,2E12.4)')
     &    'POQCHK:DEBUG:INCREASING PRECISION OF TESTS TO',DDREL,DDABS
        GOTO 50
      ENDIF
      END
