C
C
      SUBROUTINE SAMPRB(ECMI,IP,ISAM,JSAM,KSAM,LSAM,MSAM)
C********************************************************************
C
C     routine to sample number of cut graphs of different kind
C
C     input:  IP      scattering particle combination
C             ECMI    CMS energy
C             IP      -1         initialization
C                     -2         output of statistics
C                     others     sampling of cuts
C
C     output: ISAM    number of soft Pomerons cut
C             JSAM    number of soft Reggeons cut
C             KSAM    number of hard Pomerons cut
C             LSAM    number of triple/loop-Pomerons diffractively cut
C             MSAM    number of double-Pomerons cut
C
C     PRBDIS has to be called before
C
C********************************************************************
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      SAVE
C
      PARAMETER (ZERO   =  0.D0,
     &           ONE    =  1.D0,
     &           TWO    =  2.D0,
     &           OHALF  =  0.5D0,
     &           TINY   =  1.D-10)
      PARAMETER (NMAXD=100)
      COMMON /DEBUG/ IDEB(NMAXD),KSPOM,KHPOM,KSREG,KHDIR,KACCEP,
     &  KSTRG,KHTRG,KSLOO,KHLOO,KSDPO,KHDPO,KEVENT,KSOFT,KHARD
C
      CHARACTER*8 MDLNA
      COMMON /MODELS/ MDLNA(50),ISWMDL(50),PARMDL(200),IPAMDL(200)
C
      COMMON /PROCES/ IPROCE,IDNODF,IDIFR1,IDIFR2,IDDPOM,IPRON(15)
      COMMON /HADVAL/ ECMN,PCMN,SECM,SPCM,XPSUB,XTSUB,
     &                IDEQP(2),IDEQB(2),IHFLD(2,2),IHFLS(2)
C
      COMMON /CUTOFF/ PTCUT(4),CUTMU(4),FPS(4),FPH(4),PSOMIN,XSOMIN
      PARAMETER(IEETAB=20,IIMAX=30,KKMAX=10,LLMAX=2,MMMAX=1)
      REAL PROB
      COMMON /PROBAB/ PROB(4,IEETAB,0:IIMAX,0:KKMAX),EPTAB(4,IEETAB),
     &                IEEMAX,IMAX,KMAX
C  global energy
      COMMON /GLOCMS/ ECM,PCM,PMASS(2),PVIRT(2),IFPAP(2),IFPAB(2)
C  CM system of Pomeron/Reggeon exchange
      COMMON /POMCMS/ ECMP,PCMP,PMASSP(2),PVIRTP(2),GAMBEP(4),
     &                SIDP,CODP,SIFP,COFP,NPOSP(2),
     &                IDPDG1,IDBAM1,IDPDG2,IDBAM2
C  global cross sections
      COMMON /XSECTP/ SIGTOT,SIGELA,SIGVM(0:4,0:4),SIGINE,SIGDIR,
     &                SIGLSD(2),SIGHSD(2),SIGLDD,SIGHDD,SIGCDF,
     &                SIGPOM,SIGREG,SIGHAR,SIGTR1,SIGTR2,SIGLOO,SIGDPO,
     &                SIG1SO,SIG1HA,SLOEL,SLOVM(4,4),SIGGEN(4),
     &                FSUP(2)
C
      DIMENSION ECMSUM(4),CALLS(4),
     &          AVERI(4),AVERJ(4),AVERK(4),AVERL(4),AVERM(4),
     &          DEVI(4),DEVK(4),DEVL(4),DEVM(4),AVERB(0:2),AVERC(0:2)
C
*     DIMENSION IHSPOM(4),IHHPOM(4),IHAPOM(4)
*     CHARACTER*72 TITLE
C
C  sample number of interactions
      IF(IP.GE.0) THEN
        KLAST = 0
        ITER = 0
        ECMX = ECMI
        KLIM = 1
        IF((IPAMDL(13).GT.0).AND.(IPROCE.EQ.1)) THEN
	  ECMX = SECM
          KLIM = 0
        ENDIF
C  sample up to kinematic limits only
        IMAX1 = MIN(IMAX,INT(0.4D0*ECMX/CUTMU(1)))
        IF(IMAX1.LT.1) THEN
          ISAM = 0
          JSAM = 1
          KSAM = 0
          LSAM = 0
          MSAM = 0
          GOTO 150
        ENDIF
C  find interpolation factors
        IF(ECMX.LE.EPTAB(IP,1)) THEN
          I1 = 1
          I2 = 1
        ELSE IF(ECMX.LT.EPTAB(IP,IEEMAX)) THEN
          DO 50 I=2,IEEMAX
            IF(ECMX.LE.EPTAB(IP,I)) GOTO 200
 50       CONTINUE
 200      CONTINUE
          I1 = I-1
          I2 = I
        ELSE
          WRITE(6,'(/1X,A,2E12.3)')
     &      'SAMPRB:WARNING:TOO HIGH ENERGY',ECMX,EPTAB(IP,IEEMAX)
          CALL POPREV(-1)
          I1 = IEEMAX
          I2 = IEEMAX
        ENDIF
        FAC2 = 0.D0
        IF(I1.NE.I2)
     &    FAC2=LOG(ECMX/EPTAB(IP,I1))/LOG(EPTAB(IP,I2)/EPTAB(IP,I1))
        FAC1=1.D0-FAC2
        PREG = -PROB(IP,I1,0,0)*FAC1-PROB(IP,I2,0,0)*FAC2
C  calculate suppression factor
        FSUPP = PARMDL(35)**2
     &         /((PVIRTP(1)+PARMDL(35))*(PVIRTP(2)+PARMDL(35)))
C
 10     CONTINUE
        ITER = ITER+1
        PRO0 = PROB(IP,I1,0,KLAST)*FAC1+PROB(IP,I2,0,KLAST)*FAC2
        PRO0 = MAX(0.D0,PRO0)
        XI = RNDM(FAC2)*(ONE-PRO0)+PRO0
        DO 260 KSAM=KLAST,KMAX
          DO 270 ISAM=0,IMAX
            PRO = PROB(IP,I1,ISAM,KSAM)*FAC1
     &           +PROB(IP,I2,ISAM,KSAM)*FAC2
            IF(PRO.GT.XI) GOTO 100
 270      CONTINUE
 260    CONTINUE
        ISAM = MIN(IMAX,ISAM)
        KSAM = MIN(KMAX,KSAM)
        KSAM = 0
        LSAM = 0
        MSAM = 0
C
 100    CONTINUE
*       IF(ITER.EQ.1) KLAST = MIN(1,KSAM)
        IF((IP.EQ.1).AND.(ITER.EQ.1)) THEN
          AVERB(0) = AVERB(0)+1.D0
          AVERB(1) = AVERB(1)+DBLE(ISAM)
          AVERB(2) = AVERB(2)+DBLE(KSAM)
        ENDIF
        IF(ITER.GT.100) THEN
          ISAM = 0
          JSAM = 1
          KSAM = 0
          LSAM = 0
          MSAM = 0
          IF(IDEB(12).GE.3) WRITE(6,'(1X,A,I10,E11.3,I6)')
     &      'SAMPRB:WARNING:REJECTION (EV,ECM,ITER)',KEVENT,ECMX,ITER
        ELSE
C  reggeon contribution
          JSAM = 0
          IF(IPAMDL(2).EQ.1) THEN
            DO 90 I=1,ISAM
              IF(RNDM(PRO).LT.PREG) JSAM = JSAM+1
 90         CONTINUE
            ISAM = ISAM-JSAM
          ENDIF
C  limitation given by field dimensions
          IF((2*ISAM+JSAM+3*KSAM).GT.50) GOTO 10
          IF(ISWMDL(14).EQ.0) THEN
            IF(LSAM+MSAM.NE.0) GOTO 10
          ENDIF
C  reweight multiple soft according to virtualities
          IF(IPAMDL(114).GE.1) THEN
            WGX = 1.D0
            IF(IP.EQ.1) THEN
              I = MAX(0,ISAM+JSAM)
              WGX = FSUPP**I
              IF(WGX.LT.RNDM(ECMX)) GOTO 10
            ENDIF
          ENDIF
C  phase space limitation
 120      CONTINUE
*         IF(ISAM+JSAM+KSAM.EQ.0) ISAM = 1
            XM = DBLE(2*ISAM+JSAM)*CUTMU(1)
     &          +DBLE(2*LSAM+2*MSAM)*CUTMU(2)
     &          +DBLE(2*KSAM)*PTCUT(IP)
            PACC = EXP(PARMDL(9)*(CUTMU(IP)-XM)/ECMX)
            IF(RNDM(XM).GT.PACC) THEN
              IF(ISAM+JSAM+KSAM.GT.1) THEN
                IF(JSAM.GT.0) THEN
                  JSAM = JSAM-1
                  GOTO 120
                ELSE IF(ISAM.GT.0) THEN
                  ISAM = ISAM-1
                  GOTO 120
                ELSE IF(KSAM.GT.KLIM) THEN
                  KSAM = KSAM-1
                  GOTO 120
                ENDIF
              ENDIF
            ENDIF
        ENDIF
C
        ISAM = ISAM+JSAM/2
        JSAM = MOD(JSAM,2)
 150    CONTINUE
        ECMSUM(IP) = ECMSUM(IP)+ECMX
        AVERI(IP) = AVERI(IP) + DBLE(ISAM)
        AVERJ(IP) = AVERJ(IP) + DBLE(JSAM)
        AVERK(IP) = AVERK(IP) + DBLE(KSAM)
        AVERL(IP) = AVERL(IP) + DBLE(LSAM)
        AVERM(IP) = AVERM(IP) + DBLE(MSAM)
        DEVI(IP)  = DEVI(IP)  + DBLE((ISAM+JSAM)**2)
        DEVK(IP)  = DEVK(IP)  + DBLE(KSAM**2)
        DEVL(IP)  = DEVL(IP)  + DBLE(LSAM**2)
        DEVM(IP)  = DEVM(IP)  + DBLE(MSAM**2)
        CALLS(IP) = CALLS(IP) + 1.D0
*       CALL FILHIS(DBLE(ISAM),1.D0,IHSPOM(IP))
*       CALL ADDHIS(IHSPOM(IP))
*       CALL FILHIS(DBLE(KSAM),1.D0,IHHPOM(IP))
*       CALL ADDHIS(IHHPOM(IP))
*       CALL FILHIS(DBLE(ISAM+KSAM),1.D0,IHAPOM(IP))
*       CALL ADDHIS(IHAPOM(IP))
        IF(IP.EQ.1) THEN
          AVERC(0) = AVERC(0)+1.D0
          AVERC(1) = AVERC(1)+DBLE(ISAM)
          AVERC(2) = AVERC(2)+DBLE(KSAM)
        ENDIF
C
        IF(IDEB(12).GE.10) THEN
          WRITE(6,'(1X,A,2E11.4,5I3)')'SAMPRB:DEBUG:(ECM,I,J,K,L,M)',
     &      ECM,ECMX,ISAM,JSAM,KSAM,LSAM,MSAM
        ENDIF
C
C  initialize statistics
      ELSE IF(IP.EQ.-1) THEN
        DO 60 I=1,4
          ECMSUM(I) = ZERO
          AVERI(I) = ZERO
          AVERJ(I) = ZERO
          AVERK(I) = ZERO
          AVERL(I) = ZERO
          AVERM(I) = ZERO
          DEVI(I)  = ZERO
          DEVK(I)  = ZERO
          DEVL(I)  = ZERO
          DEVM(I)  = ZERO
          CALLS(I) = ZERO
*         CALL NEWHIS(0.5D0,15.5D0,ZERO,CALLS,15,IHSPOM(I))
*         CALL NEWHIS(0.5D0,15.5D0,ZERO,CALLS,15,IHHPOM(I))
*         CALL NEWHIS(0.5D0,20.5D0,ZERO,CALLS,20,IHAPOM(I))
 60     CONTINUE
        DO 65 I=0,2
          AVERB(I) = 0.D0
          AVERC(I) = 0.D0
 65     CONTINUE
        RETURN
C
C  write out statistics
      ELSE IF(IP.EQ.-2) THEN
        DO 70 I=1,4
          IF(CALLS(I).LT.2.D0) GOTO 75
          CALLS(I) = MAX(CALLS(I),ONE)
          WRITE(6,'(/,1X,A,I3,1PE13.3,0PF12.0,/,1X,A)')
     &      'SAMPRB:INTERACTION STATISTICS (IP,<ECM>,CALLS)',
     &      I,ECMSUM(I)/CALLS(I),CALLS(I),
     &      '============================='
          AVERI(I) = AVERI(I)/CALLS(I)
          AVERJ(I) = AVERJ(I)/CALLS(I)
          AVERK(I) = AVERK(I)/CALLS(I)
          AVERL(I) = AVERL(I)/CALLS(I)
          AVERM(I) = AVERM(I)/CALLS(I)
          DEVI(I)  = SQRT(DEVI(I)/CALLS(I)-(AVERI(I)+AVERJ(I))**2)
          DEVK(I)  = SQRT(DEVK(I)/CALLS(I)-AVERK(I)**2)
          DEVL(I)  = SQRT(DEVL(I)/CALLS(I)-AVERL(I)**2)
          DEVM(I)  = SQRT(DEVM(I)/CALLS(I)-AVERM(I)**2)
C
          WRITE(6,'(1X,A,I3/,4X,A,/,1X,1P6E11.3)')
     &    'AVERAGED NUMBER OF INTERACTIONS:',I,
     &' (Pom / Pom-h / Reg / enh-tri-loop / enh-dble / sum):',
     &     AVERI(I),AVERK(I),AVERJ(I),AVERL(I),AVERM(I),
     &     AVERI(I)+AVERK(I)+AVERJ(I)+AVERL(I)+AVERM(I)
          WRITE(6,'(1X,A,/,4X,A,/,1X,1P4E11.3)')
     &     'STANDARD DEVIATION ( SQRT(SIGMA) ):',
     &' (Pomeron / Pomeron-h / enh-tri-loop / enh-dble):',
     &     DEVI(I),DEVK(I),DEVL(I),DEVM(I)
C
          IF(I.EQ.1) THEN
            WRITE(6,'(1X,A)') 'AVERAGE I,K (bare)'
            WRITE(6,'(1X,F12.0,1P2E12.4)') AVERB(0),AVERB(1)/AVERB(0),
     &        AVERB(2)/AVERB(0)
            WRITE(6,'(1X,A)') 
     &        'AVERAGE I,K (with energy/virtuality corrections)'
            WRITE(6,'(1X,F12.0,1P2E12.4)') AVERC(0),AVERC(1)/AVERC(0),
     &        AVERC(2)/AVERC(0)
          ENDIF
C
*         TITLE = 'DISTRIBUTION OF SAMPLED SOFT CUT POMERONS'
*         CALL OUTHIS(IHSPOM(I),1,TITLE,ONE,0)
*         TITLE = 'DISTRIBUTION OF SAMPLED HARD CUT POMERONS'
*         CALL OUTHIS(IHHPOM(I),1,TITLE,ONE,0)
*         TITLE = 'DISTRIBUTION OF SAMPLED CUT POMERONS'
*         CALL OUTHIS(IHAPOM(I),1,TITLE,ONE,0)
 75       CONTINUE
 70     CONTINUE
        RETURN
      ENDIF
      END
