C
C
      DOUBLE PRECISION FUNCTION XDMASS(XMIN,XMAX,PREF2,IMODE)
C**********************************************************************
C
C     sampling of Mx diffractive mass distribution within
C              limits XMIN, XMAX
C
C     input:    XMIN,XMAX     limitations (GeV)
C               PREF2         original particle mass/ reference mass
C                             (squared, GeV**2)
C               IMODE         1 critical Pomeron
C                             2 supercritical Pomeron
C
C     output:   diffractive mass (GeV)
C
C**********************************************************************
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      SAVE
C
      PARAMETER (NMAXD=100)
      COMMON /DEBUG/ IDEB(NMAXD),KSPOM,KHPOM,KSREG,KHDIR,KACCEP,
     &  KSTRG,KHTRG,KSLOO,KHLOO,KSDPO,KHDPO,KEVENT,KSOFT,KHARD
C
      CHARACTER*8 MDLNA
      COMMON /MODELS/ MDLNA(50),ISWMDL(50),PARMDL(200),IPAMDL(200)
C
      PARAMETER(EPS  = 1.D-10,
     &          IONE = 1,
     &          PI   = 3.14159265359D0,
     &          PI2  = 6.28318530718D0,
     &          ZERO = 0.D0)
C
      IF((XMIN.GE.XMAX).OR.(XMIN.LE.0.3D0)) THEN
        WRITE(6,'(/1X,A,3E12.4)') 'XDMASS:ERROR:INVALID MASS LIMITS',
     &    XMIN,XMAX,PREF2
        CALL POPREV(-1)
        XDMASS = 0.135D0
        RETURN
      ENDIF
C
      IF(IMODE.GT.0) THEN
        PM2 = ZERO
      ELSE
        PM2 = PREF2
      ENDIF
C
C  critical Pomeron
      IF(ABS(IMODE).EQ.1) THEN
        XMIN2 = LOG(XMIN**2-PM2)
        XMAX2 = LOG(XMAX**2-PM2)
        XI = (XMAX2-XMIN2)*RNDM(PM2)+XMIN2
        XMA2 = EXP(XI)+PM2
C
C  supercritical Pomeron
      ELSE IF(ABS(IMODE).EQ.2) THEN
        DDELTA = 1.D0-PARMDL(48)
        XMIN2 = (XMIN**2-PM2)**DDELTA
        XMAX2 = (XMAX**2-PM2)**DDELTA
        XI = (XMAX2-XMIN2)*RNDM(PM2)+XMIN2
        XMA2 = XI**(1.D0/DDELTA)+PM2
      ELSE
        WRITE(6,'(/,1X,A,I3)') 'XMASS2:ERROR:UNSUPPORTED MODE IMODE ',
     &       IMODE
        CALL POABRT
      ENDIF
C
      XDMASS = SQRT(XMA2)
C  debug output
      IF(IDEB(43).GE.5) THEN
        WRITE(6,'(1X,A,4E12.3)') 'XDMASS:XMIN,XMAX,PREF2,XDMASS',
     &    XMIN,XMAX,PREF2,XDMASS
      ENDIF
      END
